###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from ork import dep, host, path, git, cmake, make, env
from ork.deco import Deco
from ork.wget import wget
import ork.command
from ork.command import Command
from ork import log

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _gcc

deco = Deco()

###############################################################################

class m68k_amiga_gcc(dep.StdProvider):

  def __init__(self): ############################################
    super().__init__("m68k_amiga_gcc")
    self.toolchain_dir = path.prefix()/"opt"/"toolchain"/"m68k-amiga"
    self._archlist = ["x86_64"]
    pass

  def __str__(self):
    return "Amiga-68k-Gcc"

  ########

  def env_init(self):
    log.marker("registering AmigaGCC SDK")
    env.append("PATH",self.toolchain_dir/"bin")

  ########

  def build(self): ##########################################################
    gcc = _gcc.context(self)
    return gcc.build( target="m68k-elf",
                      program_prefix="m68k-elf-amiga-",
                      install_prefix=self.toolchain_dir )==0

  ########

  def install(self): ##########################################################
    return True



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, command, path

###############################################################################

class csvparser(dep.StdProvider):
  name = "csvparser"
  def __init__(self):
    super().__init__(csvparser.name)
    src_root = self.source_root
    #################################################
    self._builder = dep.BinInstaller(csvparser.name)
    self._builder.install_item(self.source_root/"csv.h",path.includes()/"csv.h")
    #################################################
    self.declareDep("pkgconfig")
    self.declareDep("cmake")

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=csvparser.name,
                             repospec="ben-strasser/fast-cpp-csv-parser",
                             revision="master",
                             shallow=False,
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"csv.h").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"csv.h").exists()


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, log
###############################################################################
class _audiofile_from_source(dep.StdProvider):
  name = "audiofile"
  def __init__(self):
    super().__init__(_audiofile_from_source.name)
    self.VERSION = "master"
    self._builder = self.createBuilder(dep.AutoConfBuilder)
    self._builder._needsautogendotsh = True
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=_audiofile_from_source.name,
                             repospec="wtay/audiofile",
                             revision=self.VERSION,
                             recursive=False)
  ########################################################################
###############################################################################
class _audiofile_from_homebrew(dep.HomebrewProvider):
  name = "audiofile"
  def __init__(self):
    super().__init__(_audiofile_from_homebrew.name,_audiofile_from_homebrew.name)
    self.VERSION = "audiofile"
###############################################################################
class audiofile(dep.switch(linux=_audiofile_from_source, \
                           macos=_audiofile_from_homebrew)):
  def __init__(self):
    super().__init__()



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################


import os, sys, pathlib, argparse, multiprocessing

as_main = (__name__ == '__main__')

Path = pathlib.Path

curwd = Path(os.getcwd())

parser = argparse.ArgumentParser(description='ork.build environment launcher')
parser.add_argument('--launch', metavar="launchdir", help='launch from pre-existing folder' )
parser.add_argument('--prjdir', metavar="prjdir", help='override project directory' )
parser.add_argument('--chdir', metavar="chdir", help='working directory of command' )
parser.add_argument('--stack', metavar="stackdir", help='stack env' )
parser.add_argument('--prompt', metavar="prompt", help='prompt suffix' )
parser.add_argument("--command", metavar="command", help="execute in environ")
parser.add_argument("--numcores", metavar="numcores", help="numcores for environment")
parser.add_argument("--quiet", action="store_true", help="no output")
parser.add_argument('--novars', action="store_true", help='do not set env vars' )
parser.add_argument('--subspace', metavar="subspace", help='subspace to launch' )
parser.add_argument('--init', action="store_true" )
parser.add_argument('--compose',action='append',help='compose obt project into container')

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###########################################
IsQuiet = (args["quiet"]==True)
IsCommandSet = args["command"]!=None
###########################################

if IsQuiet:
    os.environ["OBT_QUIET"]="1"
#else:
#    os.environ["OBT_QUIET"]=IsQuiet

if args["prompt"]!=None:
  os.environ["OBT_USE_PROMPT_PREFIX"] = args["prompt"]

###########################################

file_path = os.path.realpath(__file__)
file_dir = os.path.dirname(file_path)
par2_dir = os.path.dirname(file_dir)
par3_dir = os.path.dirname(par2_dir)
par4_dir = os.path.dirname(par3_dir)
par5_dir = os.path.dirname(par4_dir)

root_dir = Path(par2_dir)
scripts_dir = root_dir/"scripts"
sys.path.append(str(scripts_dir))

###########################################

project_dir = root_dir
if args["prjdir"]!=None:
  project_dir = Path(args["prjdir"])

###########################################

ORK_PROJECT_NAME = "obt"
if "ORK_PROJECT_NAME" in os.environ:
  ORK_PROJECT_NAME = os.environ["ORK_PROJECT_NAME"]
OBT_STAGE = curwd/".staging"
if "OBT_STAGE" in os.environ:
  OBT_STAGE = Path(os.environ["OBT_STAGE"])
if args["launch"]!=None:
  try_staging = Path(args["launch"]).resolve()
elif args["stack"]!=None:
  try_staging = Path(args["stack"]).resolve()

NumCores = multiprocessing.cpu_count()
if args["numcores"]!=None:
  NumCores = int(args["numcores"])
if "OBT_NUM_CORES" not in os.environ:
  os.environ["OBT_NUM_CORES"]=str(NumCores)

if try_staging!=None:
  OBT_STAGE = try_staging

###########################################

os.environ["OBT_SEARCH_EXTLIST"] = ".cpp:.c:.cc:.h:.hpp:.inl:.qml:.m:.mm:.py:.txt:.glfx"

###########################################

import ork.deco
import ork.env
import ork.path
import ork.host
import ork.subspace
import ork.sdk
import ork._globals as _glob
import ork.command
import ork.subspace

deco = ork.deco.Deco()
bin_dir = root_dir/"bin"

##########################################

import ork._envutils 
envsetup = ork._envutils.EnvSetup(stagedir=OBT_STAGE,
                                  rootdir=root_dir,
                                  projectdir=project_dir,
                                  bindir=bin_dir,
                                  scriptsdir=scripts_dir,
                                  disable_syspypath=True,
                                  is_quiet=IsQuiet,
                                  project_name = ORK_PROJECT_NAME)

os.environ["OBT_STAGE"] = str(OBT_STAGE)

###########################################

if args["compose"] != None:
  for item in args["compose"]:
    envsetup.importProject(Path(item))

###########################################
# later init...
###########################################

import ork.dep

###########################################
# per dep dynamic env init
###########################################

def dynamicInit():
  ####################################
  hostinfo = ork.host.description()
  if hasattr(hostinfo,"env_init"):
    hostinfo.env_init()
  ####################################
  sdkitems = ork.sdk.enumerate()
  #print(sdkitems)
  for sdk_module_key in sdkitems.keys():
    sdk_module_item = sdkitems[sdk_module_key]
   # print(sdk_module_item)
    sdk_module = sdk_module_item._module
    #print(sdk_module)
    sdkinfo = sdk_module.sdkinfo()
    if hasattr(sdkinfo,"env_init"):
      sdkinfo.env_init()
  ####################################
  depitems = ork.dep.DepNode.FindWithMethod("env_init")
  for depitemk in depitems:
    depitem = depitems[depitemk]
    depitem.env_init()
  ####################################
  subspaceitems = ork.subspace.findWithMethod("env_init")
  for subitemk in subspaceitems:
    subitem = subspaceitems[subitemk]
    print(subitem)
    subitem._module.env_init(envsetup)
  ####################################


###########################################
if args["launch"]!=None:
###########################################
    if args["novars"]==False:
      envsetup.install()
    #############
    envsetup.lazyMakeDirs()
    envsetup.genBashRc(try_staging/".bashrc")
    dynamicInit()
    try_staging_sh = try_staging/".launch_env"
    envsetup.log(try_staging_sh)
    assert(try_staging_sh.exists())
    #############
    shell = "bash"
    bashrc = try_staging/".bashrc"
    #############
    if args["subspace"]!=None:
        if args["chdir"]!=None:
            os.chdir(args["chdir"])
        #rval = os.system(args["command"]) # call shell with new vars (just "exit" to exit)
        subspacemodulename = args["subspace"]
        ork._globals.setOption("subspacemodulename",subspacemodulename)

        subspacemodule = ork.subspace.requires(subspacemodulename)

        if args["command"]!=None:
          if args["chdir"]!=None:
            os.chdir(args["chdir"])
          rval = subspacemodule.launch(ork.command.procargs(args["command"]))
        else:
          rval = subspacemodule.shell()
  
        sys.exit(rval>>8)
    #############
    elif args["command"]!=None:
        if args["chdir"]!=None:
            os.chdir(args["chdir"])
        rval = os.system(args["command"]) # call shell with new vars (just "exit" to exit)
        sys.exit(rval>>8)
    #############
    else:
        ork.command.Command([shell,"--init-file",bashrc],environment={}).exec()
###########################################
elif args["stack"]!=None:
###########################################
    ork.env.append("OBT_STACK","<")
    if args["novars"]==False:
      envsetup.install()
    #############
    envsetup.lazyMakeDirs()
    envsetup.genBashRc(try_staging/".bashrc-stack")
    dynamicInit()
    #############
    if args["compose"]!=None:
      for item in args["compose"]:
        envsetup.importProject(Path(item)/"obt.project")
    #############
    if args["chdir"]!=None:
      os.chdir(args["chdir"])
    #############
    shell = "bash"
    bashrc = try_staging/".bashrc-stack"
    envsetup.log(deco.inf("System is <"+os.name+">"))
    print("Stacking env<%s>" % deco.val(try_staging))
    envsetup.log("ork.build eviron initialized OBT_ROOT<%s>"%deco.path(root_dir))
    if args["command"]!=None:
        ork.command.Command([shell,"--init-file",bashrc,"-c",args["command"]],environment={}).exec()
    else:
        ork.command.Command([shell,"--init-file",bashrc],environment={}).exec()
    pass
###########################################
else:
###########################################
    assert(False)



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, path

###############################################################################

class ocio(dep.StdProvider):
  name = "ocio"
  def __init__(self): ############################################
    super().__init__(ocio.name)
    self.declareDep("cmake")
    self.declareDep("pkgconfig")
    self.declareDep("oiio")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVars({
      "CMAKE_CXX_FLAGS": "-Wno-error=deprecated",
      "USE_NUKE": "OFF",
      "USE_PYTHON": "OFF",
      "OIIO_BUILD_TOOLS": "ON",
      "OIIO_BUILD_TESTS": "ON"
    })
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=ocio.name,
                             repospec="AcademySoftwareFoundation/OpenColorIO",
                             revision="v2.0.1",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"ocioconvert").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, path, command, env

###############################################################################

class parsertl14(dep.StdProvider):
  name = "parsertl14"
  def __init__(self):
    super().__init__(parsertl14.name)
    #################################################
    srcroot = self.source_root
    #################################################
    class Builder(dep.BaseBuilder):
      def __init__(self,name):
        super().__init__(name)
      def build(self,srcdir,blddir,wrkdir,incremental):
        return dep.require(self._deps)
      def install(self,blddir):
        return command.run([ "cp",
                             "-r",
                             srcroot/"include"/"parsertl",
                             path.includes()
                             ],do_log=True)==0
    #################################################
    self._builder = Builder(parsertl14.name)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=parsertl14.name,
                             repospec="tweakoz/parsertl14",
                             revision="master",
                             recursive=False)

  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"parsertl"/"parse.hpp").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, path
from ork.command import Command

###############################################################################

class nss(dep.StdProvider):
  name = "nss"
  def __init__(self):
    super().__init__(nss.name)
    self.setSourceRoot(path.builds()/"nss"/"source")
    ###########################################
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    
    nssdir = path.builds()/"nss"
    nss_source_dir = nssdir/"source"
    nss_dist_dir = nssdir/"dist"
    nss_nspr_dir = nssdir/"nspr"

    prep_command = Command(["./build.sh"],working_dir=nss_source_dir)
    clean_command = Command(["make"],working_dir=nss_source_dir)
    make_command = Command(["make"],working_dir=nss_source_dir)


    self._builder._cleanbuildcommands += [prep_command,clean_command,make_command]
    self._builder._incrbuildcommands += [prep_command,make_command]
    #assert(False) # WIP

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=nss.name,
                             repospec="nss-dev/nss",
                             revision="NSS_3_63_BRANCH",
                             recursive=False)



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, path, host
###############################################################################
class glm(dep.StdProvider):
  name = "glm"
  def __init__(self):
    super().__init__(glm.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)

    if host.IsDarwin:
      self._builder.setCmVars({
        "CMAKE_CXX_FLAGS": "-Wno-deprecated-declarations -Wno-poison-system-directories",
      })
    #self._builder.requires(["lapack"])

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=glm.name,
                             repospec="tweakoz/glm",
                             revision="master",
                             recursive=True)

  #######################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return None

  def compileenv(self): ##########################################################
    return {
        "INCLUDE_PATH": path.includes()/"glm"/"glm.hpp"
    }



#!/usr/bin/env python3

import os, sys, pathlib, argparse, shlex, subprocess, pprint
import ork.deco
import ork.path
import ork.pathtools
import ork.git
from ork.command import run
import json

lxdir = ork.path.builds()/"litex"

parser = argparse.ArgumentParser(description='ork.build environment creator')
parser.add_argument('--create', action="store_true", help='create litex environment' )
parser.add_argument('--enter', action="store_true", help='enter litex environment' )

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

if args["create"]:
  ork.pathtools.mkdir(lxdir,clean=True)
  lxdir.chdir()
  cmd1 = ["wget","https://raw.githubusercontent.com/enjoy-digital/litex/e0e9311cebcfa7334b867041850f5aca2dc50f05/litex_setup.py"]
  cmd2 = ["chmod","+x","litex_setup.py"]
  cmd3 = ["./litex_setup.py","init","install","--user"]
  run(cmd1)
  run(cmd2)
  run(cmd3)

elif args["enter"]:
  #lxdir.chdir()
  run(["bash"])



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from ork import dep, host, path, pathtools, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

deco = Deco()

VERSION = "4.7.0"
###############################################################################

class opencv(dep.StdProvider):
  name = "opencv"
  def __init__(self): ############################################
    super().__init__(opencv.name)
    self.python_dep = dep.require(["pkgconfig","pybind11","opencv_contrib"])
    self.EXR = self.declareDep("openexr")
    self.python_dep = dep.require("python")

    if self.python_dep == None:
      return False 

    self._builder = self.createBuilder(dep.CMakeBuilder)

    cmakeEnv = {
      "CMAKE_BUILD_TYPE": "RELEASE",
      "INSTALL_C_EXAMPLES": "ON",
      "INSTALL_PYTHON_EXAMPLES": "ON",
      "ENABLE_PRECOMPILED_HEADERS": "OFF",
      "WITH_TBB": "OFF",
      "WITH_GDAL": "OFF",
      "WITH_QT": "OFF",
      "WITH_GTk": "ON",
      "WITH_GTK_2_X": "OFF",
      "WITH_OPENGL": "OFF",
      "WITH_CAROTENE": "OFF",
      "WITH_FFMPEG": "OFF",
      "WITH_GSTREAMER": "OFF",
      "OPENCV_EXTRA_MODULES_PATH": "../../opencv_contrib/modules",
      "WITH_OPENEXR": "OFF",
      "BUILD_opencv_gapi":"OFF", # fails to build on ub22-aarch64
      "BUILD_opencv_python2":"OFF", # fails to build on ub22-aarch64
      "BUILD_EXAMPLES": "OFF",
      #"OPENEXR_ROOT": path.stage(),
      # todo get internal python3 working
      # todo get internal openexr working
      "PYTHON_DEFAULT_EXECUTABLE": self.python_dep.executable,
      "PYTHON3_EXECUTABLE": self.python_dep.executable,
      "PYTHON3_LIBRARY": self.python_dep.library_file,
      "PYTHON3_INCLUDE_PATH": self.python_dep.include_dir,
      "PYTHON3_PACKAGES_PATH": self.python_dep.site_packages_dir,
    }
    if host.IsLinux:
      cmakeEnv["WITH_V4L"]="ON"
    self._builder.setCmVars(cmakeEnv)

  ########################################################################

  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=opencv.name,
                             repospec="tweakoz/opencv",
                             revision=VERSION,
                             recursive=False)

  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"opencv4"/"opencv2"/"core.hpp").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from ork import dep, host, path
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

deco = Deco()

###############################################################################

class irrlicht(dep.Provider):

  def __init__(self): ############################################
    super().__init__("irrlicht")
    self.manifest = path.manifests()/"irrlicht"
    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "irrlicht (homebrew)"

  ########

  def provide(self): ##########################################################
    if False==self.OK:
      if host.IsOsx:
        self.OK = 0==Command(["brew","install","irrlicht"]).exec()
      if self.OK:
        self.manifest.touch()

    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, multiprocessing 
from ork import target, osrelease

class hostinfo:
  def __init__(self):
    self.architecture = "aarch64"
    self.os = "linux"
    host_info = osrelease.descriptor()
    self.distribution = host_info.id
    self.revision = host_info.version_id
    self.codename = host_info.version_codename
    ###################################
    aarch64_linux = target.descriptor('aarch64','linux')
    targets = set()
    targets.add(aarch64_linux)
    self.targets = targets
    self.target = aarch64_linux # host SDK
    ###################################

  @property
  def is_debian(self):
    apt_exists = os.path.exists("/etc/apt")
    return apt_exists
  @property
  def num_cores(self):
    return multiprocessing.cpu_count()
  @property
  def identifier(self):
    return "aarch64-linux"



#
#conda install pytorch torchvision torchaudio cudatoolkit=11.3 -c pytorch -y
#git clone git@github.com:pytorch/examples.git
#cd examples/mnist
#pip install -r requirements.txt
#python main.py

#cd examples/vae
#pip install -r requirements.txt
#python main.py

from ork import dep, path, command, host, wget, pathtools, subspace
from ork.deco import Deco
import ork.module
import time, re, socket, os, sys
from pathlib import Path
deco = ork.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

class subspaceinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self._name = "conda"
      self._prefix = path.subspace_root()/"conda"
      self._manifest_path = path.manifests()/self._name
    ###############################################
    @property 
    def conda_executable(self):
      return self._prefix/"bin"/"conda"
    ###############################################
    @property
    def envdir(self):
      return self._prefix/"envs"
    ###############################################
    # build conda subspace
    ###############################################
    def build(self,build_args,do_wipe=False):
      print( deco.yellow("Building Conda subspace") )
      if do_wipe:
        self._wipe()
      ########################################
      if host.IsLinux:
        if host.IsX86_64:
          url = "https://repo.anaconda.com/archive/Anaconda3-2022.05-Linux-x86_64.sh"
          MD5SUM = "a01150aff48fcb6fcd6472381652de04"
        elif host.IsAARCH64:
          url = "https://repo.anaconda.com/archive/Anaconda3-2022.05-Linux-aarch64.sh"
          MD5SUM = "7e822f5622fa306c0aa42430ba884454"
      elif host.IsOsx:
        if host.IsX86_64:
          url = "https://repo.anaconda.com/archive/Anaconda3-2022.05-MacOSX-x86_64.sh"
          MD5SUM = "5319de6536212892dd2da8b70d602ee1"
        elif host.IsAARCH64:
          url = "https://repo.anaconda.com/archive/Anaconda3-2022.05-MacOSX-arm64.sh"
          MD5SUM = "24d985d2d380c51364d4793eb1840d29"
      ########################################

      _archive_path = wget.wget(urls=[url],
                     md5val=MD5SUM,
                     output_name="anaconda.sh")

      print(_archive_path)

      if _archive_path==None:
        assert(False)
      else:
        command.run(["rm","-rf",self._prefix])
        command.run(["chmod","ugo+x",_archive_path])
        command.run([_archive_path,"-b","-p",self._prefix])

        OK = self.launch(conda_cmd="config",
                           launch_args=["--system","--set", "env_prompt", '"({default_env})|"'])==0
        if OK:
          self._manifest_path.touch()
        return OK 

    ###############################################
    # launch subprocess in conda subspace
    ###############################################
    def launch(self,working_dir=None,
                    container=None,
                    conda_cmd="run",
                    do_log=False,
                    launch_args=[]):

      conda_cmdlist = [self.conda_executable,conda_cmd]
      if do_log:
        conda_cmdlist += ["--no-capture-output"]

      if container!=None:
        conda_cmdlist += ["--name",container]
      if len(launch_args)==0:
        assert(False)

      environ = self._gen_environment(container)

      conda_cmdlist += launch_args
      print(conda_cmdlist)
      os.environ["LD_LIBRARY_PATH"]=""
      return command.run(conda_cmdlist,working_dir=working_dir,environment=environ)

    ###############################################
    def _gen_sysprompt(self,container=None):
      return "🐍-conda" if (container==None) else "🐍-%s"%container
    ###############################################
    def _gen_environment(self,container=None):
      TEMP_PATH = path.temp()
      PYTHON_HOME = self._prefix
      if container!=None:
        PYTHON_HOME = self._prefix/"envs"/container

      PYTHON_DEP = dep.instance("python")

      SITE_PKG = PYTHON_HOME/"lib"/PYTHON_DEP._deconame/"site-packages"

      pypath = os.environ["OBT_SCRIPTS_DIR"]
      pypath += ":"+str(SITE_PKG)
      pypath += ":"+os.environ["PYTHONPATH"]

      ldlibpath = str(PYTHON_HOME/"lib")
      ldlibpath += ":"+os.environ["LD_LIBRARY_PATH"]

      the_environ = {
        "LD_LIBRARY_PATH": ldlibpath,
        "OBT_PYTHON_SUBSPACE_BUILD_DIR": PYTHON_HOME/"builds",
        "OBT_SUBSPACE_LIB_DIR": PYTHON_HOME/"lib",
        "OBT_SUBSPACE_DIR": PYTHON_HOME,
        "OBT_SUBSPACE_BIN_DIR": PYTHON_HOME/"bin",
        "PYTHONPATH": pypath,
        "OBT_PYTHONHOME": PYTHON_HOME,
        "OBT_PYPKG": SITE_PKG,
        "CONDA_PREFIX": self._prefix,
        "OBT_SUBSPACE": "conda" if (container==None) else container,
        "OBT_SUBSPACE_PROMPT": self._gen_sysprompt(container=container)
      }        
      return the_environ
    ###############################################
    def env(self,args=[],
                 working_dir=None,
                 container=None,
                 do_log=False):
      return self.launch(conda_cmd="env",
                         launch_args=args,
                         working_dir=working_dir,
                         container=container,
                         do_log=do_log)
    ###############################################
    def run(self,launch_args=[],
                 working_dir=None,
                 container=None,
                 do_log=False):
      return self.launch(conda_cmd="run",
                         launch_args=launch_args,
                         working_dir=working_dir,
                         container=container,
                         do_log=do_log)
    ###############################################
    # launch conda subspace shell
    ###############################################
    def shell(self,working_dir=None,container=None):

      conda_cmdlist = [self.conda_executable,"run","--no-capture-output"]

      TEMP_PATH = path.temp()
      PYTHON_HOME = self._prefix
      if container!=None:
        conda_cmdlist += ["--name",container]
        PYTHON_HOME = self._prefix/"envs"/container

      conda_cmdlist += ["bash"]
      
      print(conda_cmdlist)

      import ork._envutils 
      sub_name = os.environ["OBT_PROJECT_NAME"]
      sub_env = ork._envutils.EnvSetup(project_name=sub_name)

      override_sysprompt = "🐍-conda" if (container==None) else "🐍-%s"%container

      BASHRC = sub_env.genBashRc()
      BASHRC += "\n\n"
      BASHRC += "%s --stack\n"%(self._prefix/"bin"/"activate")

      fname = None

      rval = 0

      environ = self._gen_environment(container)

      import tempfile
      with tempfile.NamedTemporaryFile(dir=path.temp(),mode="w",delete=False) as tempf:
        fname = tempf.name
        tempf.write(BASHRC)
        tempf.close()
        print(fname)

        conda_cmdlist += ["--rcfile",fname, "-i"]



        rval = command.run(conda_cmdlist,working_dir=working_dir,environment=environ,do_log=True)

      return rval

    ###############################################
    def _command(self,args,conda_cmd="run"):
      return self.launch(args,conda_cmd=conda_cmd)
    ###############################################
    def _wipe(self):
      print( deco.yellow("Wiping Conda subspace") )
      command.run(["rm",self._manifest_path],do_log=True)
      command.run(["rm","-rf",self._prefix],do_log=True)
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "name": "conda",
        "prefix": self._prefix,
      }
    ###############################################




#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, string, argparse
import ork.search
import ork.path
import ork.dep
import ork.deco
deco = ork.deco.Deco()

parser = argparse.ArgumentParser(description='build all box products')
parser.add_argument('--dep', help='dep to search' )
parser.add_argument('--find', help='text to search' )
parser.add_argument('--replace', help='text to replace' )

_args = vars(parser.parse_args())

#################################################################################
find_text = _args["find"]
replace_text = _args["replace"]
assert(type(find_text)==str)
assert(type(replace_text)==str)
#################################################################################
class visitor:
  def __init__(self,find,repl):
    self.by_file = dict()
    self.find = find
    self.repl = repl
  def __del__(self):
    self.process()
#######################
  def onPath(self,pth):
    self._current_root = pth
    print("/////////////////////////////////////////////////////////////")
    print("// replacing! path : %s" % pth)
    print("/////////")
#######################
  def onItem(self,item):
    if item.path not in self.by_file:
      self.by_file[item.path] = list()
    self.by_file[item.path].append(item)
#######################
  def process(self):
    for filename in self.by_file.keys():
      items = self.by_file[filename]
      #print("%s" % (filename))
      if os.path.isfile(filename):
        lines = list()
        with open(filename,"r") as file:
          lines = file.readlines()
          for item in items:
            lineno = item.lineno
            line = lines[lineno-1]
            line = line.replace(self.find,self.repl)
            lines[lineno-1] = line
            #print(line.replace("\n",""))
            pathstr = str(item.path)
            pathstr = pathstr.replace(str(self._current_root),"")
            deco_path = "%-*s"%(72,deco.path(pathstr))
            deco_lino = "%s %s"%(deco.magenta("Line"),deco.yellow(item.lineno))
            deco_lino = "%-*s"%(37,deco_lino)
            deco_text = deco.val(item.text.strip())
            print("%s%s %s" % (deco_path, deco_lino, deco_text))
        with open(filename,"w") as file:
          file.writelines(lines)
#################################################################################
if _args["dep"]!=None:
  depname = _args["dep"]
  path_list = [ork.path.builds()/depname]
  ########################
  depnode = ork.dep.DepNode.FIND(depname)
  depinst = depnode.instance
  ########################
  # allow dep module to override default search path
  ########################
  if hasattr(depinst,"find_paths"):
    path_list = depinst.find_paths()
  ########################
  #ork.search.execute_at(words,path_list)
  #########################
  ork.search.visit(find_text, visitor(find_text,replace_text),path_list=path_list)



#!/usr/bin/env python3

from ork import path, pathtools, utils
import os

build_dir = path.builds()/"litex_zephyrtest"
src_dir = os.path.dirname(os.path.realpath(__file__))

pathtools.mkdir(build_dir,clean=True)
pathtools.chdir(build_dir)
os.system("cmake %s"%str(this_dir))
os.system("make -j %d"%utils.num_cores)



VERSION = "19.0"

from ork import dep, host, path, git, pathtools, command, patch, env
from ork import log

class houdini(dep.Provider):
  def __init__(self): ############################################
    super().__init__("houdini")
    #print(options)
    self.hfs_vers = "hfs%s"%VERSION
    self.hfs_path = path.Path("/opt")/self.hfs_vers
    self.source_root = path.builds()/"houdini"
    self.build_dest = self.source_root
    self.header_dest = path.prefix()/"include"/"houdini"
    self.manifest = path.manifests()/self.hfs_vers
    self.OK = True

  def env_init(self):
    if self.hfs_path.exists():
      log.marker("registering Houdini SDK @ %s" % self.hfs_path)
      env.set("HFS",self.hfs_path)
      env.append("PATH",self.hfs_path/"bin")



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, multiprocessing
from ork import target

class hostinfo:
  ###################################
  def __init__(self):
    self.architecture = "aarch64"
    self.os = "macos"
    self.distribution = "macos"
    self.revision = platform.mac_ver()[0]
    ###################################
    self.codename = "mojave"
    sliced = int(self.revision[0:2])
    if(sliced==11):
      self.codename = "bigsur"
    if(sliced==12):
      self.codename = "monterey"
    if(sliced==13):
      self.codename = "ventura"
    ###################################
    x86_64_macos = target.descriptor('x86_64','macos')
    aarch64_macos = target.descriptor('aarch64','macos')
    aarch64_ios = target.descriptor('aarch64','ios')
    aarch64_android = target.descriptor('aarch64','android')
    ###################################

    targets = set()

    targets.add(x86_64_macos)
    targets.add(aarch64_macos)
    targets.add(aarch64_ios)
    targets.add(aarch64_android)

    self.targets = targets
    self.target = aarch64_macos # host SDK

  ###################################
  @property
  def num_cores(self):
    return multiprocessing.cpu_count()
  ###################################
  @property
  def identifier(self):
    return "aarch64-macos"
    



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v2.1"

import os, tarfile
from yarl import URL
from ork import dep, host, path, git, pathtools, patch
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command
import ork.host
import fileinput

deco = Deco()

###############################################################################

class luajit(dep.StdProvider):
  name = "luajit"
  def __init__(self): ############################################
    super().__init__(luajit.name)
    self._builder = dep.CustomBuilder(luajit.name)
    bdir = self.source_root

    cmdlist = ["make","-j",host.NumCores]

    if ork.host.IsOsx:
        cmdlist += ["MACOSX_DEPLOYMENT_TARGET=10.15"]

    clean_cmd = Command(cmdlist+["clean"],working_dir=bdir)
    install_cmd = Command(cmdlist+["install"],working_dir=bdir)

    self._builder._cleanbuildcommands += [clean_cmd,install_cmd]
    self._builder._incrbuildcommands = [install_cmd]
    self._builder._builddir = bdir

  ########

  def __str__(self):
    return "LuaJit (luajit.org-source-%s)" % VERSION

  ########################################################################
  @property
  def _fetcher(self):
    makefile_items = dict()
    makefile_items["export PREFIX= /usr/local"]="export PREFIX=%s"%path.prefix()
    patch_dict = { self.source_root/"Makefile": makefile_items }
    return dep.GithubFetcher(name=luajit.name,
                             repospec="LuaJIT/LuaJIT",
                             revision=VERSION,
                             recursive=False,
                             patchdict=patch_dict)

  ########

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"luajit-2.1").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, command, path

###############################################################################

class sse2neon(dep.StdProvider):
  name = "sse2neon"
  def __init__(self):
    super().__init__(sse2neon.name)
    #self._deps = ["pkgconfig"]
    src_root = self.source_root
    #################################################
    tgt_desc = self._target
    self._builder = self.createBuilder(dep.NopBuilder)

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=sse2neon.name,
                             repospec="tweakoz/sse2neon",
                             revision="master",
                             recursive=True)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"Makefile").exists()

  def areRequiredBinaryFilesPresent(self):
    return (self.source_root/"Makefile").exists()



from ork import dep, path, command, docker, wget, pathtools, pip, _envutils
from ork.deco import Deco
import ork.module
import time, re, socket, os, sys
from pathlib import Path
deco = ork.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

class subspaceinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self._name = "nnsvs"
      self.python = dep.DepNode.nodeForName("python").instance
      self.venv_dir = self.python.virtualenv_dir/"nnsvs"
      self._manifest_path = path.manifests()/self._name
    ###############################################
    # build the NNSVS namespace
    ###############################################
    def build(self,build_args,do_wipe=False):
      print("build nnsvs")
      venv_exists = self.venv_dir.exists()
      if do_wipe or not venv_exists:
        pathtools.rmdir(self.venv_dir,force=True)
        cmd_list = [self.python.executable,"-m","venv",self.venv_dir]
        command.run(cmd_list,do_log=True)
      dep.require("_nnsvs")
      pathtools.chdir(path.builds()/"_nnsvs")
      def pip_install(package):
        cmd_list = [self.venv_dir/"bin"/"pip3","install",package]
        command.run(cmd_list,do_log=True)
      pip_install(".")
      pip_install("git+https://github.com/tweakoz/pysinsy.git@master#egg=pysinsy")
    ###############################################
    # launch NNSVS namespace
    #  print out connection info
    ###############################################
    def launch(self,launch_args):
      print("launch nnsvs")
      #cmd_list = [self.venv_dir/"bin"/"activate"]
      #command.run(cmd_list,do_log=True)
    ###############################################
    def _gen_sysprompt(self,container=None):
      return "🎤-NNSVS" if (container==None) else "🎤-%s"%container
    ###############################################
    def _gen_environment(self,container=None):
      TEMP_PATH = path.temp()
      PYTHON_HOME = self.venv_dir
      if container!=None:
        PYTHON_HOME = self.venv_dir/"envs"/container

      PYTHON_DEP = dep.instance("python")

      SITE_PKG = PYTHON_HOME/"lib"/PYTHON_DEP._deconame/"site-packages"

      pypath = os.environ["OBT_SCRIPTS_DIR"]
      pypath += ":"+str(SITE_PKG)
      pypath += ":"+os.environ["PYTHONPATH"]

      ldlibpath = str(PYTHON_HOME/"lib")
      ldlibpath += ":"+os.environ["LD_LIBRARY_PATH"]

      the_environ = {
        "LD_LIBRARY_PATH": ldlibpath,
        "OBT_PYTHON_SUBSPACE_BUILD_DIR": PYTHON_HOME/"builds",
        "OBT_SUBSPACE_LIB_DIR": PYTHON_HOME/"lib",
        "OBT_SUBSPACE_DIR": PYTHON_HOME,
        "OBT_SUBSPACE_BIN_DIR": PYTHON_HOME/"bin",
        "PYTHONPATH": pypath,
        "OBT_PYTHONHOME": PYTHON_HOME,
        "OBT_PYPKG": SITE_PKG,
        "CONDA_PREFIX": self.venv_dir,
        "OBT_SUBSPACE": "nnsvs" if (container==None) else container,
        "OBT_SUBSPACE_PROMPT": self._gen_sysprompt(container=container),
        "NNSVS_ROOT": path.builds()/"_nnsvs",
        "NNSVS_SCRIPTS": path.modules()/"subspace"/"nnsvs",
      }        
      return the_environ
    ###############################################
    def shell(self,working_dir=None,container=None):
      print("launch nnsvs")
      #venv_py = self.venv_dir/"bin"/"python3"
      #cmd_list = [venv_py]
      #command.run(cmd_list,do_log=True)
      sub_name = os.environ["OBT_PROJECT_NAME"]
      sub_env = _envutils.EnvSetup(project_name=sub_name)

      override_sysprompt = self._gen_sysprompt(container=container)

      BASHRC = sub_env.genBashRc()
      BASHRC += "\n\n"
      BASHRC += "source %s --stack\n"%(self.venv_dir/"bin"/"activate")

      fname = None

      rval = 0

      environ = self._gen_environment(container)

      import tempfile
      with tempfile.NamedTemporaryFile(dir=path.temp(),mode="w",delete=False) as tempf:
        fname = tempf.name
        tempf.write(BASHRC)
        tempf.close()
        print(fname)

        conda_cmdlist  = ["bash"]
        conda_cmdlist += ["--rcfile",fname, "-i"]



        rval = command.run(conda_cmdlist,working_dir=working_dir,environment=environ,do_log=True)
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "name": "nnsvs",
      }
    ###############################################




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from ork import dep, host, path, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _gcc

deco = Deco()

###############################################################################

class avr_gcc(dep.Provider):

  def __init__(self): ############################################
    super().__init__("avr_gcc")
    self.manifest = path.manifests()/"avr_gcc"
    self.OK = self.manifest.exists()
    self._archlist = ["x86_64"]


  ########

  def __str__(self):
    return "Avr GCC (source)"

  ########

  def wipe(self):
    os.system("rm -rf %s"%self.source_root)
    os.system("rm -rf %s"%self.build_dest)

  ########

  def provide(self): ##########################################################
    if True: #False==self.OK:
      binutils = dep.require("avr_binutils")
      gcc = _gcc.context("gcc-avr")
      bdest = gcc.build_dir/".build"
      pfx = path.prefix()
      os.mkdir(bdest)
      os.chdir(bdest)

      cmd = Command(['../configure',
                     '--prefix=%s'%path.prefix(),
                     '--target=avr',
                     '--enable-languages=c,c++',
                     '--disable-nls',
                     '--disable-libssp',
                     '--with-dwarf2'
                     '--with-ld=%s'%(pfx/"bin"/"avr-ld"),
                     '--with-as=%s'%(pfx/"bin"/"avr-as"),
                     '--disable-shared',
                     '--disable-threads',
                     '--disable-libgomp',
                     '--libdir=%s'%(pfx/"lib"/"avr-gcc"/gcc.version),
                    ])

      cmd.exec()
      make.exec()
      make.exec("install",parallelism=0.0)

      self.OK = True
      self.manifest.touch()

      return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from ork import dep, host, path, pathtools, git, cmake, make, command
from ork.deco import Deco
from ork.wget import wget

deco = Deco()

VERSION = "v2.1.0"
###############################################################################

class fluidsynth(dep.Provider):

  def __init__(self): ############################################
    super().__init__("fluidsynth")
    self.manifest = path.manifests()/"fluidsynth"
    self.OK = self.manifest.exists()
    self.source_root = path.builds()/"fluidsynth"
    self.build_dest = self.source_root/".build"
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "CALF (github-%s)" % VERSION

  ########

  def build(self): #############################################################

    self.OK = False

    os.system("rm -rf %s"%self.source_root)

    git.Clone("https://github.com/FluidSynth/fluidsynth",
              self.source_root,
              VERSION)

    pathtools.mkdir(self.build_dest,clean=True)
    pathtools.chdir(self.build_dest)

    cmakeEnv = {
        "CMAKE_BUILD_TYPE": "RELEASE",
        "BUILD_SHARED_LIBS": "ON",
    }

    cmake_ctx = cmake.context(root="..",env=cmakeEnv)
    if cmake_ctx.exec()==0:
        if make.exec("install")==0:
            self.manifest.touch()
            self.OK = True

    return self.OK

  ########

  def provide(self): ##########################################################

    if self.should_build:
      self.OK = self.build()
    print(self.OK)
    return self.OK



import ork, os, sys

VER = "10.1.0"
HASH = "7d48e00245330c48b670ec9a2c518291"

NEWLIB_VER = "3.3.0"
NEWLIB_HASH = "af1c64d25eb3f71dec5ad7ec79877d7f"

class context:
    def __init__(self,provider,gccpatches=[],newlibpatches=[]):
        self.version = VER
        self.name = "gcc-%s" % VER
        self.xzname = "%s.tar.xz" % self.name
        self.archive_file = ork.path.downloads()/self.xzname
        self.url = "https://ftp.gnu.org/gnu/gcc/gcc-%s/%s"%(VER,self.xzname)
        self.extract_dir = ork.path.builds()/("gcc-%s"%provider)
        self.build_dir = self.extract_dir/self.name
        self.arcpath = ork.dep.downloadAndExtract([self.url],
                                                   self.xzname,
                                                   "xz",
                                                   HASH,
                                                   self.extract_dir)


        self.newlib_xzname = "newlib-%s.tar.gz" % NEWLIB_VER
        self.newlib_extract_dir = ork.path.builds()/("newlib-%s" % provider)

        self.newlib_arc = ork.dep.downloadAndExtract(["ftp://sourceware.org/pub/newlib/%s"%self.newlib_xzname],
                                                      self.newlib_xzname,
                                                      "gz",
                                                      NEWLIB_HASH,
                                                      self.newlib_extract_dir)

        ###################################3
        # patch gcc
        ###################################3
        os.chdir(self.extract_dir)
        os.system("dir")
        for item in gccpatches:
          print("apply patch<%s>"%item)
          cmd = ["patch","-p0","-i",item]
          ork.command.run(cmd)

        ###################################3
        # patch newlib
        ###################################3
        os.chdir(self.newlib_extract_dir)
        os.system("dir")
        for item in newlibpatches:
            print("apply patch<%s>"%item)
            cmd = ["patch","-p0","-i",item]
            ork.command.run(cmd)

    #############################################

    def build(self,
              target=None,
              variant="newlib",
              languages="c,c++",
              enable_set=set(),
              disable_set=set(),
              install_prefix=ork.path.prefix(),
              program_prefix=None,
              with_ld=None,
              with_as=None,
              with_build_sysroot=None,
              with_dwarf2=False
              ):

        assert(target!=None)
        self.target = target
        ######################################

        if program_prefix==None:
            program_prefix = self.target+"-"

        ######################################

        def set2opts(pfx,the_set):
            opts = list()
            for item in list(the_set):
                opts += ["%s%s"%(pfx,item)]
            return opts

        ######################################
        # required options
        ######################################

        base_build_opts = [ '--prefix=%s'%install_prefix,
                            '--target=%s'%self.target ]
        base_build_opts += ['--enable-languages=c,c++']
        base_build_opts += ['--prefix=/']

        ######################################

        build_sysroot_opts = []

        if with_build_sysroot!=None:
            build_sysroot_opts += ['--with-build-sysroot=%s'%with_build_sysroot]

        if with_ld!=None:
            base_build_opts += ['--with-ld=%s'%with_ld]
        if with_as!=None:
            base_build_opts += ['--with-as=%s'%with_as]
        if with_build_sysroot!=None:
            base_build_opts += ['--with-build-sysroot=%s'%with_build_sysroot]
        if with_dwarf2:
            base_build_opts += ['--with-dwarf2']

        base_build_opts += build_sysroot_opts
        ######################################
        # variant - newlib, default(stdc++, etc)
        ######################################


        if variant=="newlib":

            do_gcc = True
            do_newlib = True

            #######################

            if do_gcc:
                os.chdir(self.build_dir)
                ork.command.run(["rm","-rf","libstdc++-v3"])

            enable_set = enable_set | set(["threads=single","multilib"])
            disable_set = disable_set | set(["shared","libssp","libatomic",
                                             "libgomp","libmudflap","libquadmath",
                                             "nls","tls", "libstdcxx"])

            #######################

            base_build_opts += ['--with-newlib'] # dont build with or against glibc
            base_build_opts += set2opts("--disable-",disable_set)
            base_build_opts += set2opts("--enable-",enable_set)

            #######################
            # STAGE 1
            #######################

            if do_gcc:
                stg1_opts = ['--with-headers=%s'%str(self.newlib_extract_dir/"newlib"/"libc"/"include")] # no system headres
                stg1_opts += ['--program-prefix=%s-'%target]
                stg1_opts += ["--disable-libgcc"]
                os.environ["NEWLIB"] = str(self.newlib_extract_dir)
                os.chdir(self.extract_dir)
                os.system( "ln -s ${NEWLIB}/newlib .")
                os.system( "ln -s ${NEWLIB}/libgloss .")
                build_dest = self.build_dir/".build-stage1"
                if not self._build( base_build_opts + stg1_opts, build_dest, install_prefix ):
                  return False

            #######################
            # build newlib
            #######################

            if do_newlib:
              env = os.environ
              env["PATH"] = env["PATH"]+":"+str(install_prefix/"bin")
              nlbd = self.newlib_extract_dir/".build"
              os.system("rm -rf %s"%nlbd)
              os.mkdir(nlbd)
              os.chdir(nlbd)
              if ork.command.run([ "../newlib-%s/configure"%NEWLIB_VER,
                                  "--target=%s"%target,
                                  "--prefix=/",
                                  "--disable-newlib-supplied-syscalls",
                                  "--enable-multilib"] + build_sysroot_opts,environment=env)!=0:
                return False
              if ork.command.run(["make"])!=0:
                return False
              if ork.command.system(["make","DESTDIR=%s"%install_prefix,"install"])!=0:
                return False

            #######################
            # STAGE 2
            #######################

            #--with-local-prefix=/tools # gcc search path prefix
            #--with-native-system-header-dir=/tools/include # gcc headers search path


            if do_gcc:
                stg2_opts = ['--program-prefix=%s'%program_prefix]
                stg2_opts += ["--enable-libgcc"]
                stg2_opts += ["--with-newlib"]
                stg2_opts += ["--with-local-prefix=%s"%install_prefix] # gcc search path prefix
                stg2_opts += ["--with-native-system-header-dir=%s"%(with_build_sysroot/"usr"/"include")] # gcc headers search path
                stg2_opts += ["--with-sysroot=%s"%(with_build_sysroot)] # gcc headers search path
                os.environ["NEWLIB"] = str(self.newlib_extract_dir)
                os.chdir(self.extract_dir)
                build_dest = self.build_dir/".build-stage2"
                #os.environ["PATH"] = os.environ["PATH"]+":"+str(install_prefix/"bin")
                if not self._build( base_build_opts + stg2_opts, build_dest, install_prefix ):
                   return False

        else: # // default, stdc++
            assert(False)
            pass

        return True

        ######################################


    #############################################

    def _build(self,build_opts,bdest,install_prefix):
        os.mkdir(bdest)
        os.chdir(bdest)
        if ork.command.run(['../configure']+build_opts)!=0:
          return False
        if ork.make.exec("all")!=0:
          return False
        if ork.command.system(["make","DESTDIR=%s"%install_prefix,"install"])!=0:
          return False
        return True



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, path, log, env, host
from ork.command import Command
###############################################################################
class ue5(dep.StdProvider):
  name = "ue5"
  def __init__(self):
    super().__init__(ue5.name)
    self._archlist = ["x86_64","aarch64"]
    self._oslist = ["Linux","Darwin"]
    self._builder = dep.CustomBuilder(ue5.name)

    bdir = self.source_root

    setupcmd = Command(["./Setup.sh"],working_dir=bdir,do_log=False)
    genprjcmd = Command(["./GenerateProjectFiles.sh"],working_dir=bdir,do_log=False)

    if host.IsOsx:
      buildcmd = Command(["xcodebuild","-workspace","UE5.xcworkspace","-scheme","UE5"],
                          working_dir=bdir,
                          do_log=False)
    elif host.IsLinux:
      buildcmd = Command(["make"],
                          working_dir=bdir,
                          do_log=False)

    self._builder._cleanbuildcommands += [setupcmd,genprjcmd,buildcmd]
    self._builder._incrbuildcommands = [buildcmd]

    self._builder._builddir = bdir/".fake_build"
    #print("unrealdep<%s>"%self)

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GithubFetcher(name=ue5.name,
                                repospec="EpicGames/UnrealEngine",
                                revision="5.0.2-release",
                                recursive=True,
                                shallow=False)
    return fetcher
  ########################################################################
  @property
  def root_dir(self):
    return self.source_root
  ########################################################################
  @property
  def engine_dir(self):
    return self.source_root/"Engine"
  ########################################################################
  @property
  def extras_dir(self):
    return self.engine_dir/"Extras"
  ########################################################################
  @property
  def host_sdk_dir(self):
    return self.extras_dir/"ThirdPartyNotUE"/"SDKs"/"HostLinux"/"Linux_x64"
  ########################################################################
  @property
  def compiler_dir(self):
    return self.host_sdk_dir/"v20_clang-13.0.1-centos7"/"x86_64-unknown-linux-gnu"
  ########################################################################

  def find_paths(self):
    return [self.source_root/"Engine"/"Plugins"] + \
           [self.source_root/"Engine"/"Source"/"Runtime"]    

  def env_init(self):
    log.marker("registering Unreal SDK")
    env.append("PATH",path.builds()/"unreal"/"Engine"/"Binaries"/"Linux")

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"GenerateProjectFiles.sh").exists()

  def areRequiredBinaryFilesPresent(self):
    return (self.source_root/"Engine"/"Binaries"/"Linux"/"UE4Editor-Linux-Debug").exists()

#${OBT_BUILDS}/ue5/Engine/Binaries/Linux/UnrealEditor -enablehighdpi



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, path

###############################################################################

class openexr(dep.StdProvider):
  name = "openexr"
  def __init__(self):
    super().__init__(openexr.name)
    self.declareDep("cmake")
    #self.declareDep("fltk")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("OPENEXR_VIEWERS_ENABLE","OFF")
    self._builder.setCmVar("CMAKE_MODULE_PATH",self.source_root)
    self._builder.setCmVar("PYILMBASE_ENABLE","OFF")
    #self._builder.requires(["fltk"])
    
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=openexr.name,
                             repospec="AcademySoftwareFoundation/openexr",
                             #revision="v2.5.8",
                             revision="v3.1.5",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libIlmImf.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v2.50.0-obt"

import os
from ork import dep, host, path, cmake, git, make, pathtools, log
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

deco = Deco()
class _realsense_from_source(dep.StdProvider):
  name = "realsense2"
  def __init__(self): ############################################
    super().__init__(_realsense_from_source.name)
    self.fullver = VERSION
    self.declareDep("cmake")
    self.declareDep("glfw")
    self.createBuilder(dep.CMakeBuilder)

    CMAKE_VARS = {
      "CMAKE_CXX_FLAGS": "-Wno-error=deprecated",
      "FORCE_RSUSB_BACKEND": "true"
    }

    if self._target.identifier=="aarch64-macos":
      _vars = {
        "TBB_ROOT": "/opt/homebrew/Cellar/tbb/2021.5.0_1/",
        "CMAKE_THREAD_LIBS_INIT": "-lpthread",
        "CMAKE_HAVE_THREADS_LIBRARY": "1",
        "CMAKE_USE_WIN32_THREADS_INIT": "0",
        "CMAKE_USE_PTHREADS_INIT": "1",
        "THREADS_PREFER_PTHREAD_FLAG": "ON"
      }
      CMAKE_VARS.update(_vars)

    self._builder.setCmVars(CMAKE_VARS)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=_realsense_from_source.name,
                             repospec="tweakoz/librealsense",
                             revision=VERSION,
                             recursive=True)

  #######################################################################
  def linkenv(self): 
    LIBS = ["librealsense2"]
    return {
        "LIBS": LIBS,
        "LFLAGS": ["-l%s"%item for item in LIBS]
    }
  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  #######################################################################
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"librealsense2.so").exists()

###############################################################################

class _realsense_from_homebrew(dep.HomebrewProvider):
  def __init__(self):
    super().__init__("librealsense","librealsense")
    self.fullver = "5.15.1"
  #def install_dir(self):
  #  return path.Path("/usr/local/opt/qt5")

###############################################################################

BASE = _realsense_from_source
#if host.IsOsx:
#  BASE = _realsense_from_homebrew

###############################################################################

class realsense2(BASE):
  def __init__(self):
    super().__init__()
  ########
  def __str__(self):
    return "realsense2"
  ########
  def env_init(self):
    log.marker("registering realsense2(%s) SDK"%self.fullver)
    #if host.IsOsx:
    #  qtdir = Path("/")/"usr"/"local"/"opt"/"qt5"
    #else:
    #  qtdir = path.stage()/"qt5"
    #env.set("QTDIR",qtdir)
    #env.prepend("PATH",qtdir/"bin")
    #env.prepend("LD_LIBRARY_PATH",qtdir/"lib")
    #env.append("PKG_CONFIG_PATH",qtdir/"lib"/"pkgconfig")
    #env.prepend("PKG_CONFIG_PATH",qtdir/"lib"/"pkgconfig")
    #env.set("QTVER",self.fullver)
  ########
  #@property
  #def include_dir(self):
  #  return path.qt5dir()/"include"



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, command, path

###############################################################################

class libcurl(dep.StdProvider):
  name = "libcurl"
  def __init__(self):
    super().__init__(libcurl.name)
    src_root = self.source_root
    #################################################
    self._builder = self.createBuilder(dep.CMakeBuilder)
    if host.IsOsx:
      import ork.macos_homebrew
      sslroot = ork.macos_homebrew.prefix_for_package("openssl")
      print(sslroot)
      self._builder.setCmVar("OPENSSL_ROOT_DIR",sslroot)
      self._builder.setCmVar("USE_ZLIB","ON")
    #################################################
    self.declareDep("pkgconfig")
    self.declareDep("cmake")
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=libcurl.name,
                             repospec="tweakoz/curl",
                             revision="obt-7.84",
                             recursive=False)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libcurl.so").exists()


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v5.2.1"

import os, tarfile
from ork import dep, host, path, git, pathtools, command, patch, env
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command
import ork.host
from ork import log

deco = Deco()

###############################################################################

class lua(dep.Provider):

  def __init__(self): ############################################
    super().__init__("lua")
    #print(options)
    self.source_root = path.builds()/"lua"
    self.build_dest = self.source_root
    self.header_dest = path.prefix()/"include"/"lua"
    self.manifest = path.manifests()/"lua"
    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "lua (lua.org-source-%s)" % VERSION

  ########

  def env_init(self):
    log.marker("registering Lua SDK")

  ########

  def download_and_extract(self): #############################################

    command.system("rm -rf %s"%self.source_root)

    git.Clone("https://github.com/lua/lua",
              self.source_root,
              rev=VERSION,
              cache=False)

    patch_items = dict()
    patch_items["CC= clang-3.8"]="CC= g++"
    patch_items["CFLAGS= -Wall -O2 $(MYCFLAGS)"]="CFLAGS= -Wall -O2 $(MYCFLAGS) -fPIC"
    if ork.host.IsLinux:
        patch_items["MYLDFLAGS= $(LOCAL) -Wl,-E"]="MYLDFLAGS= $(LOCAL) -Wl,-E -fPIC"
    else:
        patch_items["MYLDFLAGS= $(LOCAL) -Wl,-E"]="MYLDFLAGS= $(LOCAL) -fPIC"
        patch_items["-lhistory"]=""

    patch.patch_with_dict(self.source_root/"makefile",patch_items)

  def build(self): ############################################################

    self.download_and_extract()
    os.chdir(str(self.source_root))

    cmd = ["make","-j",host.NumCores]

    if ork.host.IsOsx:
        cmd += ["MACOSX_DEPLOYMENT_TARGET=10.14"]

    self.ok = (0 == Command(cmd).exec())

    return self.install()

  def install(self):
    if self.ok:
      cmd = ["cp",self.source_root/"lua",path.bin()/"lua"]
      self.ok = (0 == Command(cmd).exec())
      if self.ok:
        cmd = ["cp",self.source_root/"liblua.a",path.prefix()/"lib"/"liblua.a"]
        self.ok = (0 == Command(cmd).exec())
        if self.ok:
          pathtools.mkdir(self.header_dest,clean=True)
          cmd = ["cp", self.source_root/"*.h",str(self.header_dest)+"/"]
          self.ok = (0 == command.system(cmd))
    return self.ok 

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"makefile").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"lua").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, env, path, host
###############################################################################
class openroad(dep.StdProvider):
  name = "openroad"
  def __init__(self):
    super().__init__(openroad.name)
    self.declareDep("cmake")
    self._archlist = ["x86_64"]

    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.requires(["eigen","lemongraph"])
    if host.IsOsx:
      self._builder.setCmVar("TCL_LIBRARY",path.osx_brewopt()/"tcl-tk"/"lib"/"libtcl8.6.dylib")
  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GitFetcher(openroad.name)
    fetcher._git_url = "https://github.com/tweakoz/OpenROAD"
    fetcher._cache=False
    fetcher._recursive=True
    fetcher._revision = "toztest"
    return fetcher
  ########################################################################



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, host, path, pathtools, command, env
import os
###############################################################################
class ue5_cesium(dep.StdProvider):
  name = "ue5_cesium"
  def __init__(self):
    super().__init__(ue5_cesium.name)
    #self._archlist = ["x86_64"]
    self.declareDep("cmake")
    self.UE5 = self.declareDep("ue5")
    UE5DIR = self.UE5.source_root
    UE5ENGDIR = UE5DIR/"Engine"

    self._osenv = dict()
    self._osenv["UNREAL_ENGINE_DIR"]=UE5DIR
    self._osenv["UNREAL_ENGINE_COMPILER_DIR"]=self.UE5.compiler_dir
    self._osenv["UNREAL_ENGINE_LIBCXX_DIR"]=UE5ENGDIR/"Source"/"ThirdParty"/"Unix"/"LibCxx"

    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._osenv = self._osenv

    self._builder._cmakeenv["CMAKE_TOOLCHAIN_FILE"]="unreal-linux-toolchain.cmake"
    self._builder._cmakeenv["CMAKE_POSITION_INDEPENDENT_CODE"]="ON"
    self._builder._cmakeenv["CMAKE_BUILD_TYPE"]="Debug"

    self.build_dest = self.source_root/"extern"/"build"
    self.build_working_dir = self.source_root/"extern"

    #####################################
    # install triangle
    #####################################

    def post_install():
      #print("post installing IGL!!")
      #igl_builddir = path.builds()/"igl"/".build"
      #pathtools.copyfile(igl_builddir/"lib"/"libtriangle.a",path.libs()/"libtriangle.a")
      #pathtools.copyfile(igl_builddir/"_deps"/"triangle-src"/"triangle.h",path.includes()/"triangle.h")
      cmdlist = ["cmake","--build","build","--target","install"]
      command.run(cmdlist,working_dir=self.source_root/"extern")
      pass

    self._builder._onPostInstall = post_install

    #####################################

  ########
  def on_build_shell(self):
    print(self._osenv)
    env.set("UNREAL_ENGINE_DIR",str(self.UE5.root_dir))
    env.set("UNREAL_ENGINE_COMPILER_DIR",str(self.UE5.compiler_dir))
    env.set("UNREAL_ENGINE_LIBCXX_DIR",str(self.UE5.engine_dir/"Source"/"ThirdParty"/"Unix"/"LibCxx"))
    env.set("CESIUM_FOR_UNREAL_DIR",str(self.source_root))

    shscr = '#!/usr/bin/env sh\n'
    shscr += 'cmake -B build -S . -DCMAKE_TOOLCHAIN_FILE="unreal-linux-toolchain.cmake" -DCMAKE_POSITION_INDEPENDENT_CODE=ON -DCMAKE_BUILD_TYPE=Release\n'
    shscr +="cmake --build build --target install -j 16\n"
    #shscr +='cd $UNREAL_ENGINE_DIR/Engine/Build/BatchFiles\n'
    #shscr +='./RunUAT.sh BuildPlugin -Plugin="$CESIUM_FOR_UNREAL_DIR/CesiumForUnreal.uplugin" -Package="$CESIUM_FOR_UNREAL_DIR/../packages/CesiumForUnreal" -CreateSubFolder -TargetPlatforms=Linux\n'
    shscr +='$UNREAL_ENGINE_DIR/Engine/Build/BatchFiles/RunUAT.sh BuildPlugin -Plugin="$CESIUM_FOR_UNREAL_DIR/CesiumForUnreal.uplugin" -Package="$OBT_BUILDS/ue5pkg_CesiumForUnreal" -CreateSubFolder -TargetPlatforms=Linux\n'
    shscr +='mkdir -p $UNREAL_ENGINE_DIR/Engine/Plugins/Marketplace\n'
    shscr +='cp -r $OBT_BUILDS/ue5pkg_CesiumForUnreal $UNREAL_ENGINE_DIR/Engine/Plugins/Marketplace/\n'

    with open(os.open("%s"%str(self.build_working_dir/"build_obt.sh"), os.O_CREAT | os.O_WRONLY, 0o777), "w") as f:
      f.write(shscr)


    return command.subshell( directory=self.build_working_dir,
                             prompt = "ue5_cesium",
                             environment = dict() )

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=ue5_cesium.name,
                             repospec="CesiumGS/cesium-unreal",
                             revision="v1.14.0-ue5",
                             shallow=False,
                             recursive=True)

  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"ue5_cesium"/"igl_inline.h").exists()

"""
  560  obt.dep.build.py ue5_cesium --incremental 
  561  cd extern/cesium-native/extern/
  562  ls
  563  cd asyncplusplus/
  564  ls
  565  make
  566  ls -l
  567  vim Makefile 
  568  make clean
  569  mak
  570  make
  571  make clean
  572  VERBOSE=1 make
  573  pwd
  574  ls -l
  575  vim Makefile 
  576  fgrep -R relink *
  577  make pre-install
  578  vim Makefile 
  579  make Async++/preinstall
  580  ls -l
  581  ls CMakeFiles/CMakeRelink.dir/

  .. then for spdlog

"""

"""
ERROR: Failed to copy 
       <STAGING>/.staging-temp/builds/ue5_cesium/extern/cesium-native/extern/KTX-Software/other_lib/mac/Release/SDL2.framework/Versions/Current/Frameworks/hidapi.framework/hidapi 
    to <STAGING>/.staging-temp/builds/packages/CesiumForUnreal/HostProject/Plugins/CesiumForUnreal/extern/cesium-native/extern/KTX-Software/other_lib/mac/Release/SDL2.framework/Versions/Current/Frameworks/hidapi.framework/hidapi

ERROR: Failed to copy <STAGING>/.staging-temp/builds/ue5_cesium/extern/cesium-native/extern/KTX-Software/NOTICE.md to <STAGING>/.staging-temp/builds/packages/CesiumForUnreal/HostProject/Plugins/CesiumForUnreal/extern/cesium-native/extern/KTX-Software/NOTICE.md
       (see /home/michael/Library/Logs/Unreal Engine/LocalBuildLogs/Log.txt for full exception trace)
"""

#dlopen failed: libUnrealEditor-SunPosition.so: cannot open shared object file: No such file or directory

"""
find ../ue5 | grep libUnrealEditor-SunPosition.so
../ue5/Engine/Intermediate/Build/Linux/B4D820EA/UnrealEditor/Development/libUnrealEditor-SunPosition.so_nodebug
../ue5/Engine/Intermediate/Build/Linux/B4D820EA/UnrealEditor/Development/Link-libUnrealEditor-SunPosition.so.link.sh
../ue5/Engine/Intermediate/Build/Linux/B4D820EA/UnrealEditor/Development/libUnrealEditor-SunPosition.so.psym
../ue5/Engine/Plugins/Runtime/SunPosition/Intermediate/Build/Linux/B4D820EA/UnrealEditor/Development/SunPosition/libUnrealEditor-SunPosition.so.response
../ue5/Engine/Plugins/Runtime/SunPosition/Binaries/Linux/libUnrealEditor-SunPosition.so
"""


#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import ork._globals
import ork.subspace
from ork import dep, path
from ork.deco import Deco
deco = Deco()

###############################################################################

parser = argparse.ArgumentParser(description='ork.build subspace launcher')
parser.add_argument('subspacemodulename', metavar='S', type=str, help='a subspace module to launch')
parser.add_argument('--launchargs', action="store", type=str, help='launch arguments' )

args, unknownargs = parser.parse_known_args()

_args = vars(args)

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

subspacemodulename = _args["subspacemodulename"]

ork._globals.setOption("subspacemodulename",subspacemodulename)

subspacemodule = ork.subspace.requires(subspacemodulename)

print("unknownargs: %s"%unknownargs)
if len(unknownargs)==0:
    subspacemodule.shell()
else:
    subspacemodule.launch(unknownargs)

sys.exit(0)


import ork.xcode 

class sdkinfo:
  def __init__(self):
    self.identifier = "x86_64-macos"
    self.architecture = "x86_64"
    self.os = "macos"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-macos","aarch64-macos"]

  def env_init(self):
    _xcodesdkstr = ork.command.capture([
      "xcodebuild",
      "-version",
      "-sdk","macosx"]).splitlines()
    for l in _xcodesdkstr:
      x = l.split(": ")
      if x[0]=="Path":
        ork.env.append("OBT_MACOS_SDK_DIR",x[1])
      if x[0]=="PlatformVersion":
        ork.env.append("OBT_MACOS_PLATFORM_VERSION",x[1])




#!/usr/bin/env python3
import os, sys, string
import ork.search as search
import ork.path

#################################################################################
class visitor:
  def __init__(self,find,repl):
    self.by_file = dict()
    self.find = find
    self.repl = repl
  def __del__(self):
    self.process()
#######################
  def onPath(self,pth):
    print("/////////////////////////////////////////////////////////////")
    print("// path : %s" % pth)
    print("/////////")
#######################
  def onItem(self,item):
    if item.path not in self.by_file:
      self.by_file[item.path] = list()
    self.by_file[item.path].append(item)
#######################
  def process(self):
    for filename in self.by_file.keys():
      items = self.by_file[filename]
      if os.path.isfile(filename):
        lines = list()
        with open(filename,"r") as file:
          lines = file.readlines()
          for item in items:
            lineno = item.lineno
            line = lines[lineno-1]
            line = line.replace(self.find,self.repl)
            lines[lineno-1] = line
            print(line.replace("\n",""))
        with open(filename,"w") as file:
          file.writelines(lines)
      #print("%-*s : line %-*d : %s" % (40, item.path, 5, item.lineno, item.text)
      #pass
    pass
#################################################################################

if __name__ == "__main__":
  #########################
  if not len(sys.argv) >= 2:
    print("usage: find_word repl_word")
    sys.exit(1)
  #########################
  ork.path.project_root().chdir()
  #########################
  find_word = sys.argv[1]
  repl_word = sys.argv[2]
  #########################
  search.visit(find_word, visitor(find_word,repl_word))



#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import ork._globals
import ork.docker
from ork import dep, path
from ork.deco import Deco
deco = Deco()

def print_item(key,val):
 dstr = deco.inf(dockerid)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

###############################################################################

parser = argparse.ArgumentParser(description='ork.build docker builder')
parser.add_argument('dockermodulename', metavar='D', type=str, help='a docker module to build')
parser.add_argument('--force', action="store_true", help='force rebuild' )
parser.add_argument('--wipe', action="store_true", help='wipe and rebuild' )
parser.add_argument('--buildargs', action="store", type=str, help='wipe and rebuild' )

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

dockermodulename = _args["dockermodulename"]

ork._globals.setOption("dockermodulename",dockermodulename)

for item in "force wipe".split(" "):
  ork._globals.setOption(item,_args[item]==True)

print(dockermodulename)

dockermodule = ork.docker.descriptor(dockermodulename)

print(dockermodule)

dockermodule.build(_args["buildargs"])

sys.exit(0)


#!/usr/bin/env python3

import ork.path
import ork.litex

arty_cmd = "./arty.py --output-dir %s"%(ork.path.builds()/"artysoc")

ork.litex.run( cpu="lm32",
               platform="arty",
               target="base",
               cmd=arty_cmd )



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, path
from ork.command import Command
###############################################################################
class bazel(dep.StdProvider):
  name = "bazel"
  def __init__(self):
    super().__init__(bazel.name)
    from yarl import URL
    VER = "5.1.1"
    self.URLBASE = URL("https://releases.bazel.build/%s/release"%VER)
    self._fname = "bazel-%s-darwin-x86_64"%VER
    self._url = self.URLBASE/self._fname


    self._builder = self.createBuilder(dep.BinInstaller)

    self._builder.install_item(source=self.source_root/self._fname, 
                               destination=path.bin()/"bazel",
                               flags="ugo+x")

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.WgetFetcher(bazel.name)
    fetcher._fname = self._fname
    fetcher._url = self._url
    fetcher._arctype = "none"
    fetcher._md5 = "ed363c3e58e6fca9766bf0676ce4c53c"
    return fetcher
  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/self._fname).exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.stage()/"share"/"pkgconfig"/"bazel3.pc").exists()

  def compileenv(self): ##########################################################
    return {
        "INCLUDE_PATH": path.includes()/"bazel3"
    }







###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "f700c7790aaa030e794b52ba7791a05c085faf0c"

from ork import dep, host, path
from ork.command import Command as cmd

###############################################################################

class xatlas(dep.StdProvider):
  name = "xatlas"
  def __init__(self): ############################################
    super().__init__(xatlas.name)
    self.declareDep("premake")
    self._builder = self.createBuilder(dep.CustomBuilder)

    build_dir = self.source_root/"build"/"gmake2"
    build_bin_dir = build_dir/"bin"/"x86_64"/"Release"
    include_dir = self.source_root/"source"/"xatlas"
    shared_lib_name = "libxatlas.so"


    self._builder._incrbuildcommands = [
      cmd(["premake5","gmake2"]),
      cmd(["make","-j",host.NumCores],
          working_dir=build_dir)
    ]
    self._builder._installcommands += {
      cmd(["install",build_bin_dir/shared_lib_name,path.libs()/shared_lib_name]),
      cmd(["install",include_dir/"xatlas.h",path.includes()/"xatlas.h"]),
      cmd(["install",include_dir/"xatlas_c.h",path.includes()/"xatlas_c.h"])
    }
  ########################################################################
  def __str__(self): 
    return "xatlas (github-%s)" % VERSION
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=xatlas.name,
                             repospec="jpcy/xatlas",
                             revision=VERSION,
                             recursive=False)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"premake5.lua").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"xatlas.h").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from ork import dep, host, path, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
import ork.command
from ork.command import Command

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _gcc

deco = Deco()

###############################################################################

class rv32_gcc(dep.Provider):

  def __init__(self): ############################################
    super().__init__("rv32_gcc")
    self._archlist = ["x86_64"]
    pass

  def provide(self): ##########################################################
    gcc = _gcc.context("riscv32-elf")
    self.OK = gcc.build()


    self.OK = True

    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from ork import dep, host, path, pathtools, git, cmake, make, command
from ork.deco import Deco
from ork.wget import wget

deco = Deco()

VERSION = "2.20.2"
###############################################################################

class faust(dep.Provider):

  def __init__(self): ############################################
    super().__init__("faust")
    self.manifest = path.manifests()/"faust"
    self.OK = self.manifest.exists()
    self.source_root = path.builds()/"faust"
    self.build_dest = self.source_root/".build"
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "faust (github-%s)" % VERSION

  ########

  def build(self): #############################################################

    self.OK = False
    if self.should_force_build:
        os.system("rm -rf %s"%self.source_root)

    git.Clone("https://github.com/grame-cncm/faust",
              self.source_root,
              VERSION,
              recursive=True)

    pathtools.chdir(self.source_root)

    makeenv = {
        "PREFIX": path.stage()
    }

    if command.Command(['make',"-e"],environment=makeenv).exec()==0:
        if command.Command(['make',"-e","install"],environment=makeenv).exec()==0:
          self.OK = True
          self.manifest.touch()

    return self.OK

  ########

  def provide(self): ##########################################################

    if self.should_build:
      self.OK = self.build()
    print(self.OK)
    return self.OK



import os, tarfile
from ork import dep, host, path, make, pathtools
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

VER = "0.29.2"
HASH = "f6e931e319531b736fadc017f470e68a"

class pkgconfig(dep.Provider):

  def __init__(self): ############################################
    super().__init__("pkgconfig")
    self.scope = dep.ProviderScope.INIT
    self.manifest = path.manifests()/"pkgconfig"
    self.extract_dir = path.builds()/"pkgconfig"
    self.source_dir = self.extract_dir/("pkg-config-%s" % VER)
    self.build_dir = self.source_dir/".build"
    self.url = "http://pkgconfig.freedesktop.org/releases/pkg-config-%s.tar.gz" % VER

    self.OK = self.manifest.exists()

  def build(self): ##########################################################
    self.arcpath = dep.downloadAndExtract([self.url],
                                          "pkg-config-%s" % VER,
                                          "gz",
                                          HASH,
                                          self.extract_dir)


    self.build_dir.mkdir()
    self.build_dir.chdir()

    conf_cmd = [
      "../configure",
      "--prefix=%s"%path.prefix(),
      "--with-internal-glib"
    ]
    if host.IsLinux and host.IsAARCH64:
      conf_cmd += ["--build=aarch64-linux-gnu"]

    self.OK = (Command(conf_cmd).exec()==0)

    #assert(False)

    if host.IsLinux and host.IsX86_64:
      f2r = path.stage()/"bin"/"x86_64-unknown-linux-gnu-pkg-config"
      os.system( "rm -f %s" % f2r)
    elif host.IsLinux and host.IsAARCH64:
      f2r = path.stage()/"bin"/"x86_64-unknown-linux-gnu-pkg-config"
      os.system( "rm -f %s" % f2r)

    pathtools.ensureDirectoryExists(path.pkgconfigdir())

    if self.OK:
      self.OK = (make.exec( "install" )==0)

    return self.OK
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_dir/"config.guess").exists()
  ########################################################################
  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"pkg-config").exists()



import ork

VER = "2.34"
HASH = "664ec3a2df7805ed3464639aaae332d6"

class context:
  def __init__(self,provider):
    self._archlist = ["x86_64"]
    self.name = "binutils-%s" % VER
    self.xzname = "%s.tar.xz" % self.name
    self.archive_file = ork.path.downloads()/self.xzname
    self.url = "https://ftp.gnu.org/gnu/binutils/%s"%self.xzname
    self.extract_dir = provider.build_src
    self.build_dir = provider.build_dest
    self.arcpath = ork.dep.downloadAndExtract([self.url],
                                               self.xzname,
                                               "xz",
                                               HASH,
                                               self.extract_dir)



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from yarl import URL
from ork import dep, host, path
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command
from ork.cmake import context

deco = Deco()

###############################################################################

class gitpython(dep.Provider):

  def __init__(self): ############################################
    super().__init__("gitpython")
    self.manifest = path.manifests()/"gitpython"
    self.OK = self.manifest.exists()

  ########

  def __str__(self):
    return "gitpython (pip3)"

  ########

  def provide(self): ##########################################################
    if False==self.OK:
      self.OK = 0==Command(["pip3","install","gitpython"]).exec()
    if self.OK:
      self.manifest.touch()

    return self.OK



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, host, path
from ork.command import Command
from ork import log
from ork.deco import Deco
deco = Deco()

class _llvm_from_source(dep.StdProvider):
  name = "llvm"
  def __init__(self,name):
    super().__init__(_llvm_from_source.name)
    self._archlist = ["x86_64"]
    self._builder = self.createBuilder(dep.CMakeBuilder)
    ##########################################
    # llvm cmake file is 1 subdir deeper than usual
    ##########################################
    self.build_src = self.source_root/"llvm"
    self.build_dest = self.source_root/".build"
    ##########################################
    self._builder.setCmVars({
        "CMAKE_BUILD_TYPE": "RELEASE",
        "BUILD_SHARED_LIBS": "ON",
        "LLVM_INSTALL_UTILS": "ON",
        "LLVM_ENABLE_DUMP": "ON",
        "LLVM_ENABLE_PROJECTS": "clang;libcxx;libcxxabi"
    })

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GithubFetcher(name=_llvm_from_source.name,
                                repospec="llvm/llvm-project",
                                revision="llvmorg-12.0.1",
                                recursive=False)
    return fetcher
  ########################################################################

  def install_dir(self):
    return path.stage()

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.stage()/"bin"/"llvm-cov").exists()

###############################################################################

class _llvm_from_homebrew(dep.HomebrewProvider):
  def __init__(self,name):
    super().__init__(name,name)
  def install_dir(self):
    return path.Path("/usr/local/opt/llvm")

###############################################################################

BASE = _llvm_from_source
if host.IsOsx:
  BASE = _llvm_from_homebrew

###############################################################################

class llvm(BASE):
  def __init__(self):
    super().__init__("llvm")
  def env_init(self):
    log.marker("registering LLVM SDK")



from ork.wget import wget
from ork import path, command, env, pathtools
from yarl import URL 
import os, sys

class sdkinfo:
  ##################################################################
  def __init__(self):
    self.identifier = "aarch64-android"
    self.architecture = "aarch64"
    self.os = "android"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["aarch64-macos"]
    self.sdk_dir = path.sdks()/"android-sdk"
    self.ANDROID_VERSION="28" 
    self.ANDROID_BUILD_TOOLS_VERSION="27.0.3"
    self.ANDROID_NDK_VERSION = "20.0.5594570"

  ##################################################################
  def misc(self):
    return {
      "hello": "yo"
    }
  ##################################################################
  def env_init(self):
    env.append("ANDROID_HOME",self.sdk_dir)
    env.append("ANDROID_VERSION",self.ANDROID_VERSION)
    env.append("ANDROID_BUILD_TOOLS_VERSION",self.ANDROID_BUILD_TOOLS_VERSION)
    env.append("JAVA_HOME",'/Applications/Android Studio.app/Contents/jre/Contents/Home')
    # TODO - activate following only in SDK subshell
    env.append("PATH",path.sdks()/"cmdline-tools"/"bin")
    env.append("PATH",self.sdk_dir/"build-tools"/self.ANDROID_BUILD_TOOLS_VERSION/"bin")
    env.append("PATH",self.sdk_dir/"tools"/"bin")
    env.append("PATH",self.sdk_dir/"tools"/"bin")
    env.append("PATH",self.sdk_dir/"platform-tools")
    env.append("PATH",self.sdk_dir/"gradle-5.6.4"/"bin")

  ##################################################################
  def install(self):
    STUDIO_VERSION = "2021.2.1.14"
    dlgoogle = URL("https://dl.google.com/android/repository")
    redirector = URL("https://redirector.gvt1.com/edgedl/android/studio/install")/STUDIO_VERSION

    studio_name = "android-studio-%s-mac_arm.dmg"%STUDIO_VERSION
    ndk_name = "android-ndk-r22b-darwin-x86_64.zip"
    cltools_name = "commandlinetools-mac-8512546_latest.zip"

    gradle_path = wget(urls=["https://services.gradle.org/distributions/gradle-5.6.4-bin.zip"],
                       output_name="gradle.zip",
                       md5val="d1456582f1513c1c04d4bd4ae952f25d")

    studio_path = wget(urls=[redirector/studio_name], 
                output_name = studio_name,
                md5val = "0c2b3c364510d4941d77ab879c901859" )

    #ndk_zip_path = wget(urls=[dlgoogle/ndk_name], 
    #             output_name = ndk_name,
    #             md5val = "3e22f5eefb5f0dfaf0d779d60f85685b" )

    clt_zip_path = wget(urls=[dlgoogle/cltools_name], 
                 output_name = cltools_name,
                 md5val = "769ecd04e00367458de6e948b944c20e" )

    if (gradle_path==None) or (studio_path==None) or (clt_zip_path==None):
      print( "Could not complete downloads for Android SDK")
      sys.exit(-2)

    path.sdks().chdir()

    pathtools.ensureDirectoryExists(self.sdk_dir)
    pathtools.ensureDirectoryExists(self.sdk_dir/"licenses")
    os.system('echo "24333f8a63b6825ea9c5514f83c2829b004d1fee" > "$ANDROID_HOME/licenses/android-sdk-license"')

    command.run(["open",studio_path])
    command.run(["unzip","-o",clt_zip_path])
    command.run(["unzip","-o",gradle_path])

    command.run(["sdkmanager",
                 "--sdk_root=%s" % self.sdk_dir,
                 "--update"],
                 do_log=True)

    command.run(["sdkmanager", 
                 "--sdk_root=%s" % self.sdk_dir, 
                 "build-tools;%s" % self.ANDROID_BUILD_TOOLS_VERSION],
                 do_log=True)

    command.run(["sdkmanager",
                 "--sdk_root=%s" % self.sdk_dir, 
                 "platforms;android-%s" % self.ANDROID_VERSION],
                 do_log=True)

    command.run(["sdkmanager",
                 "--sdk_root=%s" % self.sdk_dir, 
                 "ndk;%s" % self.ANDROID_NDK_VERSION],
                 do_log=True)

    command.run(["sdkmanager", 
                 "--sdk_root=%s" % self.sdk_dir, 
                 "platform-tools"],
                 do_log=True)


    sys.exit(-1)




  ##################################################################



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from ork import dep, host, path, pathtools, git, cmake, make, command
from ork.deco import Deco
from ork.wget import wget

deco = Deco()

VERSION = "master"
###############################################################################

class calf(dep.Provider):

  def __init__(self): ############################################
    super().__init__("calf")
    self.manifest = path.manifests()/"calf"
    self.OK = self.manifest.exists()
    self.source_root = path.builds()/"calf"
    self.build_dest = self.source_root/".build"

  ########

  def __str__(self):
    return "CALF (github-%s)" % VERSION

  ########

  def build(self): #############################################################

    dep.require("fluidsynth")
    self.OK = False


    os.system("rm -rf %s"%self.source_root)

    git.Clone("https://github.com/calf-studio-gear/calf",
              self.source_root,
              VERSION)

    pathtools.chdir(self.source_root)

    os.system("aclocal --force")
    os.system("libtoolize --force --automake --copy")
    os.system("autoheader --force")
    os.system("autoconf --force")
    os.system("automake -a --copy")

    if command.Command(['./configure',
                        '--prefix=%s'%path.prefix(),
                       ]).exec()==0:
      if make.exec("all")==0:
        if make.exec("install",parallelism=0.0)==0:
          self.OK = True
          self.manifest.touch()

    return self.OK

  ########

  def provide(self): ##########################################################

    if self.should_build:
      self.OK = self.build()
    print(self.OK)
    return self.OK



#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import ork.docker

parser = argparse.ArgumentParser(description='ork.build docker information')
parser.add_argument('docker', metavar='T', type=str, help='a docker you want information on')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

dockerid = _args["docker"]

D = ork.docker.descriptor(dockerid)

from ork import dep, path
from ork.deco import Deco
deco = Deco()

def print_item(key,val):
 dstr = deco.inf(dockerid)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

print_item("info",D.info())
#print_item("c_compiler",SDK.c_compiler)
#print_item("cxx_compiler",SDK.cxx_compiler)



#!/usr/bin/env python3

from ork.command import Command

items =  ["yarl","pkgconfig","boost","luajit","gcode_gpr","postgresql"]
items += ["avr_binutils","avr_gcc","avr_libc","simavr"]
items += ["lm32_binutils","lm32_gcc"]

assert(os.environ["OBT_SUBSPACE"]=="host")

for item in items:
	Command(["obt.dep.require.py",item]).exec()



import sys, threading, time, json, random
from pathlib import Path
from yarl import URL
from orkbb import BranchStatus,WorkerStatus, DEBUGFORMAT

################################################################################
# WorkerConfig
################################################################################
class Config(object):
  ###################################################################
  def __init__(self,configfile,outputdir):
    PROJTEXT = ""
    self.outputdir= outputdir
    #######################################
    with open(configfile,"r") as f:
      PROJTEXT = f.read()
    PROJOBJ = json.loads(PROJTEXT)
    #######################################
    self.name = PROJOBJ["name"]
    self.master_ssh_host = PROJOBJ["master_ssh_host"]
    self.master_ssh_user = PROJOBJ["master_ssh_user"]
    self.master_ssh_port = PROJOBJ["master_ssh_port"]
    self.master_zmq_host = PROJOBJ["master_zmq_host"]
    self.master_zmq_port = PROJOBJ["master_zmq_port"]
    self.sleeptime = PROJOBJ["sleeptime"]
    #######################################
  #####



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, path

###############################################################################

class lexertl14(dep.StdProvider):
  name = "lexertl14"
  def __init__(self):
    super().__init__(lexertl14.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=lexertl14.name,
                             repospec="tweakoz/lexertl14",
                             revision="e7632c128f07eebc3dc8e402101cf124eb734dfe",
                             recursive=False)

  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"lexertl"/"char_traits.hpp").exists()



import ork.xcode 
import ork.command 
import ork.env
import ork.path 
import os

class sdkinfo:
  #################################################
  # env_init invoked once at container startup time
  #################################################
  def env_init(self):
    _xcodesdkstr = ork.command.capture([
      "xcodebuild",
      "-version",
      "-sdk"])
    has_iossdk15_2 = _xcodesdkstr.find("(iphoneos15.2)")>0
    has_iossdk15_5 = _xcodesdkstr.find("(iphoneos15.5)")>0
    print("has_iossdk15_2<%s>"%has_iossdk15_2)
    print("has_iossdk15_5<%s>"%has_iossdk15_5)
    if has_iossdk15_2:
      _iossdkstr = ork.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "iphoneos15.2",
        "Path"],do_log=False).splitlines()
      ork.env.append("OBT_IOS_SDK","iphoneos15.2")
      ork.env.append("OBT_IOS_SDK_DIR",_iossdkstr)
    if has_iossdk15_5:
      _iossdkstr = ork.command.capture([
        "xcodebuild",
        "-version",
        "-sdk", "iphoneos15.5",
        "Path"],do_log=False).splitlines()
      ork.env.append("OBT_IOS_SDK","iphoneos15.5")
      ork.env.append("OBT_IOS_SDK_DIR",_iossdkstr)
  #############################################
  def __init__(self):
    self.identifier = "aarch64-ios"
    self.architecture = "aarch64"
    self.os = "ios"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-macos","aarch64-macos"]
  #############################################
  @property
  def sdkdir(self):
    return ork.path.Path(os.environ["OBT_IOS_SDK_DIR"])
  #############################################
  def misc(self):
    return {
      "sdkdir": self.sdkdir
    }




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, command, pathtools, path
###############################################################################
class oneapimkl(dep.StdProvider):
  VERSION ="v0.2"
  NAME = "oneapimkl"
  def __init__(self):
    super().__init__(oneapimkl.NAME)
    self.declareDep("cmake")
    self._builder = dep.CMakeBuilder(oneapimkl.NAME)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=oneapimkl.NAME,
                             repospec="oneapi-src/oneMKL",
                             revision=oneapimkl.VERSION,
                             recursive=False)

  #######################################################################
  def on_build_shell(self):
    pathtools.mkdir(self.build_dest,clean=False)
    return command.subshell( directory=self.build_dest,
                             prompt = "ASSIMP",
                             environment = dict() )
  ########
  def linkenv(self): ##########################################################
    return {
        "LIBS": ["oneapimkl"]
    }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libassimp.so").exists()




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, path, executors
from ork.command import Command, system

###############################################################################

class tflite(dep.StdProvider):
  name = "tflite"
  def __init__(self):
    super().__init__(tflite.name)
    self.declareDep("bazel")
    PYTHON = self.declareDep("python")
    self._builder = self.createBuilder(dep.CustomBuilder)
    env = {

      "TF_NEED_CUDA":"0",
      "TF_NEED_ROCM":"0",
      "TF_CUDA_CLANG":"0",
      "TF_DOWNLOAD_CLANG":"0",
      "TF_SET_ANDROID_WORKSPACE":"0",
      "TF_CONFIGURE_IOS":"0",
      
      "CC_OPT_FLAGS":"--copt=-mavx --copt=-mavx2 --copt=-mfma --copt=-msse4.2 --copt=-mfpmath=both --config=cuda",
      "PYTHON_BIN_PATH":PYTHON.executable,
      "USE_DEFAULT_PYTHON_LIB_PATH":"1",
      "TF_NEED_JEMALLOC":"1",
      "TF_NEED_GCP":"0",
      "TF_NEED_HDFS":"0",
      "TF_ENABLE_XLA":"0",
      "TF_NEED_OPENCL":"1",


    }


    configure_command = Command([
      "./configure"],
      working_dir=self.source_root,
      environment=env)

    build_cmd = Command(
      [ "bazel", 
        "build",
        "-c","opt",
        "//tensorflow/lite:libtensorflowlite.dylib"],
      working_dir=self.source_root,
      environment=env)

    TENSORFLOW_INST_ROOT = path.stage()/"tensorflow-bin"
    TENSORFLOW_INST_LIB = TENSORFLOW_INST_ROOT/"lib"
    TENSORFLOW_INST_INC = TENSORFLOW_INST_ROOT/"include"/"tensorflow"
    LITE_BUILD_DIR = self.source_root/"bazel-bin"/"tensorflow"/"lite"

    install_commands = []

    def add_cmdlist(cmdlist):
      nonlocal install_commands
      cmd = Command(cmdlist,
                    use_shell=True,      
                    working_dir=self.source_root,
                    environment=env)
      install_commands += [cmd]

    def add_systemcmd(cmdlist):
      nonlocal install_commands
      install_commands += [lambda: system(cmdlist,working_dir=self.source_root)==0]


    install_commands += [executors.rmdir(TENSORFLOW_INST_ROOT,force=True)]
    install_commands += [executors.mkdir(TENSORFLOW_INST_LIB,parents=True,clean=True)]
    install_commands += [executors.mkdir(TENSORFLOW_INST_INC,parents=True,clean=True)]

    #add_systemcmd(["cp","-f","bazel-bin/tensorflow/lite/*.a",TENSORFLOW_INST_LIB])
    #add_systemcmd(["cp","-f","bazel-bin/tensorflow/lite/*.dylib",TENSORFLOW_INST_LIB])

    install_commands += [executors.copy_files( src_dir=LITE_BUILD_DIR,
                                               pattern="*.dylib",
                                               dst_dir=TENSORFLOW_INST_LIB,
                                               modeset="c")]

    install_commands += [executors.copy_files( src_dir=LITE_BUILD_DIR,
                                               pattern="*.a",
                                               dst_dir=TENSORFLOW_INST_LIB,
                                               modeset="c")]

    def rsync(src=None,dst=None,incs=[],excs=[]):
      nonlocal install_commands
      cmd = ["rsync","-avzh"]
      for item in incs:
        cmd += ["--include",item]
      for item in excs:
        cmd += ["--exclude",item]
      cmd += [src,dst]
      add_systemcmd(cmd)


    #rsync( --exclude '_virtual_includes/' --include '*/' --include '*.h' --include '*.inc' --exclude '*' bazel-bin/ $tensorflow_root/include/
    #rsync( src="tensorflow/cc", dst=TENSORFLOW_INST_INC, incs=['*/','*.h','*.inc'], excs=['*'] )
    #rsync( --include '*/' --include '*.h' --include '*.inc' --exclude '*' tensorflow/core $tensorflow_root/include/tensorflow/
    #rsync( --include '*/' --include '*' --exclude '*.cc' third_party/ $tensorflow_root/include/third_party/
    #rsync( --include '*/' --include '*' --exclude '*.txt' bazel-tensorflow/external/eigen_archive/Eigen/ $tensorflow_root/include/Eigen/
    #rsync( --include '*/' --include '*' --exclude '*.txt' bazel-tensorflow/external/eigen_archive/unsupported/ $tensorflow_root/include/unsupported/
    #rsync( --include '*/' --include '*.h' --include '*.inc' --exclude '*' bazel-tensorflow/external/com_google_protobuf/src/google/ $tensorflow_root/include/google/
    #rsync( --include '*/' --include '*.h' --include '*.inc' --exclude '*' bazel-tensorflow/external/com_google_absl/absl/ $tensorflow_root/include/absl/

    self._builder._cleanbuildcommands += [configure_command,build_cmd]
    self._builder._incrbuildcommands += [build_cmd]+install_commands

    #for item in install_commands:
    #  print(item.command_list)

    #cmake ../tensorflow/lite -DTFLITE_KERNEL_TEST=on -DTFLITE_ENABLE_GPU=ON
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=tflite.name,
                             repospec="tweakoz/tensorflow",
                             revision="master",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libtflitemcDynamics.so").exists()


"""
export TF_NCCL_VERSION=1.3

export PYTHONPATH=${TF_ROOT}/lib
export PYTHON_ARG=${TF_ROOT}/lib
export CUDA_TOOLKIT_PATH=${CUDA_HOME}
export CUDNN_INSTALL_PATH=${CUDA_HOME}

export TF_NEED_ROCM=0
export TF_NEED_IGNITE=0
export TF_NEED_GCP=0
export TF_NEED_CUDA=1
export TTENSORRT_INSTALL_PATH = ???
export TF_CUDA_VERSION="$($CUDA_TOOLKIT_PATH/bin/nvcc --version | sed -n 's/^.*release \(.*\),.*/\1/p')"
export TF_CUDA_COMPUTE_CAPABILITIES=6.1,5.2,3.5
export TF_NEED_HDFS=0
export TF_NEED_OPENCL=0
export TF_NEED_JEMALLOC=1
export TF_ENABLE_XLA=0
export TF_NEED_VERBS=0
export TF_CUDA_CLANG=0
export TF_CUDNN_VERSION="$(sed -n 's/^#define CUDNN_MAJOR\s*\(.*\).*/\1/p' $CUDNN_INSTALL_PATH/include/cudnn.h)"
export TF_NEED_MKL=0
export TF_DOWNLOAD_MKL=0
export TF_NEED_AWS=0
export TF_NEED_MPI=0
export TF_NEED_GDR=0
export TF_NEED_S3=0
export TF_NEED_OPENCL_SYCL=0
export TF_SET_ANDROID_WORKSPACE=0
export TF_NEED_COMPUTECPP=0
export GCC_HOST_COMPILER_PATH=$(which gcc)
export CC_OPT_FLAGS="-march=native"

export TF_NEED_KAFKA=0
export TF_NEED_TENSORRT=0"""




#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import ork.sdk

parser = argparse.ArgumentParser(description='ork.build sdk information')
parser.add_argument('sdk', metavar='S', type=str, help='a sdk you want information on')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

sdkid = _args["sdk"]

target_arch = sdkid.split('-')[0]
target_os = sdkid.split('-')[1]

S = ork.sdk.descriptor(target_arch,target_os)

if S==None:
  print("SDK not supported on hosts")
  sys.exit(-1)

if hasattr(S,"install"):
  S.install()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v5.0.0-beta2"

from ork import dep, host, path
from ork.command import Command as cmd

###############################################################################

class premake(dep.StdProvider):
  name = "premake"
  def __init__(self): ############################################
    super().__init__(premake.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CustomBuilder)
    PLATFORM = "linux"
    if host.IsDarwin:
      PLATFORM = "osx"
    self._builder._incrbuildcommands = [
      cmd(["make","-f","Bootstrap.mak",PLATFORM])
    ]
    self._builder._installcommands += {
      cmd(["install","bin/release/premake5",path.bin()/"premake5"])
    }
  ########################################################################
  def __str__(self): 
    return "premake (github-%s)" % VERSION
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=premake.name,
                             repospec="premake/premake-core",
                             revision=VERSION,
                             recursive=False)
  ########################################################################
  #def linkenv(self):
  #  LIBS = ["premake"]
  #  return {
  #      "LIBS": LIBS,
  #      "LFLAGS": ["-l%s"%item for item in LIBS]
  #  }
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"doxyfile").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"premake.h").exists()



from yarl import URL
from ork import dep, path

###############################################################################

class drawtext(dep.StdProvider):
  VERSION = "v0.5"
  name = "drawtext"
  def __init__(self):
    super().__init__(drawtext.name)
    self._archlist = ["x86_64"]
    self.mustBuildInTree()
    self._builder = self.createBuilder(dep.AutoConfBuilder)

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=drawtext.name,
                             repospec="jtsiomb/libdrawtext",
                             revision=drawtext.VERSION,
                             recursive=False)

  #######################################################################

  def areRequiredSourceFilesPresent(self):
    print(self.source_root)
    return (self.source_root/"configure").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libembree3.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION ="v2.1.0"

import os, tarfile
from ork import dep, host, path, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command
from ork.cmake import context

deco = Deco()

###############################################################################

class easyprof(dep.StdProvider):
  name = "easyprof"
  #############################################
  def __init__(self):
    super().__init__(easyprof.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=easyprof.name,
                             repospec="yse/easy_profiler",
                             revision=VERSION,
                             recursive=False)

  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libeasy_profiler.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from ork import dep, path
###############################################################################
class openvr(dep.StdProvider):
  name = "openvr"
  def __init__(self):
    super().__init__(openvr.name)
    self._archlist = ["x86_64"]
    self._oslist = ["Linux"]
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON"
    }

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=openvr.name,
                             repospec="ValveSoftware/openvr",
                             revision="v1.11.11",
                             recursive=False)
  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libopenvr_api.a").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from ork import dep, host, path, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _binutils

deco = Deco()

###############################################################################

class lm32_binutils(dep.Provider):

  def __init__(self): ############################################
    super().__init__("lm32_binutils")
    self._archlist = ["x86_64"]
    pass


  def provide(self): ##########################################################
    bu = _binutils.context("binutils-lm32")
    bdest = bu.build_dir/".build"

    os.mkdir(bdest)
    os.chdir(bdest)

    Command(['../configure',
             '--prefix=%s'%path.prefix(),
             '--program-prefix=lm32-elf-',
             '--target=lm32-elf']).exec()

    make.exec("all")
    make.exec("install",parallelism=0.0)

    self.OK = True
    return self.OK



from setuptools import setup, find_packages
import glob, os
import platform

###############################################################################

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

###############################################################################

def package_files(directory):
    data_files = []
    for root, dirs, files in os.walk(directory):
        for filename in files:
            if not (filename.endswith('.pyc') or '.egg-info' in root or '__pycache__' in root):
                filepath = os.path.join(root, filename)
                data_files.append((os.path.join('ork.build.tools', root), [filepath]))
    return data_files

###############################################################################

def package_files_platform_specific(directory):
    data_files = []
    for root, dirs, files in os.walk(directory):
        for filename in files:
            if not (filename.endswith('.pyc') or '.egg-info' in root or '__pycache__' in root):
                filepath = os.path.join(root, filename)
                if platform.system() == 'Darwin':  # This means it's macOS
                    if "obt.ix" in filepath:
                        continue
                else:  # For any other OS, we skip 'obt.osx'
                    if "obt.osx" in filepath:
                        continue
                data_files.append((os.path.join('ork.build.tools', root), [filepath]))
    return data_files

###############################################################################

def glob_platform_specific(directory):
    files = glob.glob(directory)
    outfiles = []
    for f in files:
      if platform.system() == 'Linux':
          sysmatch = 'obt.osx' not in f
      if platform.system() == 'Darwin':
          sysmatch = 'obt.ix' not in f
      if sysmatch:
        outfiles.append(f)
    return outfiles

###############################################################################

module_files = package_files('modules')
example_files = package_files('examples')
test_files = package_files('tests')
binpriv_files = package_files_platform_specific('bin_priv')

###############################################################################

setup(
    name="ork.build.tools",
    version="0.0.11",
    packages=find_packages(exclude=["bin_priv", "build", "dist"]),
    package_dir={
        "": ".",
    },
    data_files=module_files + example_files + test_files + binpriv_files,
    scripts=glob_platform_specific('bin_pub/*'),
    long_description=long_description,
    long_description_content_type='text/markdown'
)




###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "10.4"
HASH = "8e8770c289b3e0bdb779b5b171593479"

import os, tarfile
from yarl import URL
from ork import dep, host, path, cmake
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

deco = Deco()

###############################################################################

class postgresql(dep.Provider):

  def __init__(self): ############################################
    super().__init__("postgresql")
    #print(options)
    build_dest = path.builds()/"postgresql"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"postgresql"
    self.OK = self.manifest.exists()
    self.fname = "postgresql-%s.tar.bz2"%VERSION
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "Postgresql (%s-source)" % VERSION

  ########

  def download_and_extract(self): #############################################

    url = URL("https://ftp.postgresql.org/pub/source/v%s/postgresql-%s.tar.bz2"%(VERSION,VERSION))

    self.arcpath = dep.downloadAndExtract([url],
                                          self.fname,
                                          "bz2",
                                          HASH,
                                          self.build_dest)


  def build(self): ############################################################
    self.download_and_extract()
    source_dir = self.build_dest/("postgresql-%s"%VERSION)
    build_temp = source_dir/".build"
    print(build_temp)
    if build_temp.exists():
      Command(["rm","-rf",build_temp]).exec()

    build_temp.mkdir(parents=True,exist_ok=True)
    os.chdir(str(build_temp))
    Command(["../configure","--prefix",path.prefix()]).exec()
    return 0==Command(["make","-j",host.NumCores,"install"]).exec()



#!/usr/bin/env python3

import os, sys, pathlib, argparse
import ork._globals

parser = argparse.ArgumentParser(description='ork.build dep builder')
parser.add_argument('dependency', metavar='D', type=str, help='a dependency to build')
parser.add_argument('--force', action="store_true", help='force rebuild' )
parser.add_argument('--wipe', action="store_true", help='wipe and redownload' )
parser.add_argument('--nofetch', action="store_true", help='do not run download/extract/fetch step' )
parser.add_argument('--incremental', action="store_true", help='incremental rebuild' )
parser.add_argument('--serial', action="store_true", help='serial build' )
parser.add_argument('--usegitclone', action="store_true", help='do not use github wget, use github clone for fetching' )
parser.add_argument('--verbose', action="store_true", help='verbose build' )
parser.add_argument('--debug', action="store_true", help='debug build' )

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

depname = _args["dependency"]

ork._globals.setOption("depname",depname)

for item in "force wipe incremental nofetch serial usegitclone verbose debug".split(" "):
  ork._globals.setOption(item,_args[item]==True)

from ork import dep
from ork.deco import Deco
deco = Deco()

if os.environ["OBT_SUBSPACE"]!="host":
  node = dep.instance(depname)
  if not node._allow_build_in_subspaces:
    print( "Dependency %s not allowed to be built in subspaces other than host"%depname)
    sys.exit(-1)

chain = dep.Chain(depname)

#print(chain)
for item in reversed(chain._list):
  name = deco.key("%s"%(item._name))
  should = item.supports_host and item.should_build
  #print("dep<%s> ShouldBuild<%s>"%(name,deco.val("%s"%should)))
  ret = True
  if should:
    ret = item.provide()
  
  if not ret:
  	print(deco.red("Dependency Failed! : %s ret<%s>"%(name,ret)))
  	sys.exit(-1)
#print("dep<%s> returned<%s>"%(depname,status))
sys.exit(0)


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile, sys
from ork import dep, host, path, git, cmake, make
from ork.deco import Deco
from ork.wget import wget
from ork.command import Command

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

import _binutils

deco = Deco()

###############################################################################

class arm64_binutils(dep.StdProvider):

  def __init__(self): ############################################
    super().__init__("arm64_binutils")
    self._archlist = ["x86_64"]

  ########

  def __str__(self):
    return "Arm64 BinUtils (source)"

  ########

  def build(self): ##########################################################

    bu = _binutils.context(self)

    os.mkdir(self.build_dest)
    os.chdir(self.build_dest)

    toolchain_dir = path.prefix()/"opt"/"toolchain"/"aarch64-elf"

    ok = Command(['../%s/configure'%bu.name,
                  '--prefix=%s'%toolchain_dir,
                  '--target=aarch64-elf',
                  '--program-prefix=aarch64-elf-',
                  '--disable-werror',
                  '--disable-nls']).exec()==0

    return make.exec("all")==0

  def install(self): ##########################################################
    os.chdir(self.build_dest)
    return make.exec("install",parallelism=0.0)==0



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from ork import dep, path, command

###############################################################################

class flatcam(dep.StdProvider):
  name = "flatcam"
  def __init__(self):
    super().__init__(flatcam.name)
    self._builder = self.createBuilder(dep.NopBuilder)
    self._modules  = ["tk","pyqt5","reportlab","svglib","vispy==0.7","pyopengl","rtree"]
    self._modules += ["rasterio","ezdxf","ortools","serial","numpy","shapely"]
    self._modules += ["lxml","cycler","python-dateutil","kiwisolver"]
    self._modules += ["dill","svg.path","freetype-py","fontTools","simplejson","qrcode"]
    self._buildcmd = ["make","install"]
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=flatcam.name,
                             repospec="tweakoz/flatcam",
                             revision="toz-obt2",
                             recursive=True)
  ########################################################################
  def on_build_shell(self):
    command.run(["pip3","install"]+self._modules)
    command.run(self._buildcmd)
    print( "run ./assets/linux/flatcam-beta")
    return command.subshell( directory=self.source_root,
                             prompt = "FLATCAM",
                             environment = dict() )
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"setup_ubuntu.sh").exists()
  def areRequiredBinaryFilesPresent(self):
    return (self.source_root/"assets"/"linux"/"flatcam-beta").exists()



#!/usr/bin/env python3
###########################################################
# lsusbx : fancy usb device lister
#  @ 2019 Michael T. Mayers : michael@tweakoz.com
#  license: go to town...
# todo: make it faster and prettier
###########################################################
import subprocess, sys, os, re
from pathlib import Path
import pyudev
import ork.deco
deco = ork.deco.Deco()

class devinfo:
    def __init__(self,name,usbpos):
        self._name = name
        self._usbpos = usbpos
        self._devtype = ""
        self._model = ""
        self._modelenc = ""
        self._vendor = ""
        self._vendordb = ""
        self._ttyname = ""
        self._drivers = set()
        self._serial = ""
        self._speed = 0
    def vendorShort(self):
        lv = len(self._vendor)
        lvdb = len(self._vendordb)
        vendor = self._vendor
        if (lvdb>0) and (lvdb<lv) and (lv>8):
            vendor = self._vendordb
        return vendor
    def modelShort(self):
        return self._modelenc

devices = dict()

udev = pyudev.Context()

############################################################
# gather devices
############################################################

all_devices = udev.list_devices(subsystem="usb")

for d in all_devices:
   #print(dict(d))
   DEVTYPE = d.get("DEVTYPE")
   if DEVTYPE=="usb_device": # leaf devices
     busnum = d.attributes.get("busnum")
     devnum = d.attributes.get("devnum")
     devpath = d.attributes.get("devpath")
     speed = d.attributes.get("speed")
     pos = "%03d:%03d"%(int(busnum),int(devnum))
     #print(d.sys_name,busnum,devnum,devpath )
     devi = devinfo(pos,pos)
     MODELENC = d.get("ID_MODEL_ENC")
     VENDOR = d.get("ID_VENDOR_ENC")
     VENDORDB = d.get("ID_VENDOR_FROM_DATABASE")
     SERIAL = d.get("ID_SERIAL_SHORT")
     if VENDOR!=None:
       devi._vendor=bytes(VENDOR, "utf-8").decode("unicode_escape")
     if VENDORDB!=None:
       devi._vendordb=bytes(VENDORDB, "utf-8").decode("unicode_escape")
     if MODELENC!=None:
       devi._modelenc=bytes(MODELENC, "utf-8").decode("unicode_escape")
     if SERIAL!=None:
       devi._serial=bytes(SERIAL, "utf-8").decode("unicode_escape")
     devi.speed = speed
     devices[pos]=devi

############################################################
# gather interfaces
############################################################

for d in all_devices:
   DEVTYPE = d.get("DEVTYPE")
   if DEVTYPE=="usb_interface": # endpoints in device
     p = d.parent
     #print(dict(p))
     busnum = p.attributes.get("busnum")
     devnum = p.attributes.get("devnum")
     devpath = p.attributes.get("devpath")
     if busnum!=None and devnum!=None:
      pos = "%03d:%03d"%(int(busnum),int(devnum))
      assert(pos in devices)
      devi = devices[pos]
      DRIVER = d.get("DRIVER")
      MODEL = d.get("ID_MODEL_FROM_DATABASE")
      if DRIVER!=None:
         devi._drivers.add(DRIVER)
      if MODEL!=None:
         devi._model=MODEL

############################################################
# gather tty's
############################################################

for device in udev.list_devices(subsystem="tty",ID_BUS="usb"):
    #print(device)
    p = device.find_parent(subsystem="usb")
    if p!=None:
        k = device.sys_name
        pp = p.parent
        busnum = pp.attributes.get("busnum")
        devnum = pp.attributes.get("devnum")
        pos = "%03d:%03d"%(int(busnum),int(devnum))
        assert(pos in devices)
        devi = devices[pos]
        devi._ttyname = device.sys_name

############################################################

devnull = open(os.devnull, 'w')

def getlines(cmd):
    lines = subprocess.check_output(cmd,
                                    shell=True,
                                    stderr=devnull,
                                    universal_newlines=True).split("\n")
    return lines

usbitems = getlines("lsusb")

###############################
def rgb256(r,g,b):
    return deco.rgb256(r,g,b)
###############################
def reset():
    return deco.reset()
###############################
def rgbstr(r,g,b,string):
    return deco.rgbstr(r*255,g*255,b*255, str(string) )
###############################

byloc = dict()
for item in usbitems:
    columns = item.split(" ")
    if len(columns)>5:
     bus = columns[1]
     dev = columns[3][0:3]
     ifclass = ""

     ##############################
     # this is the code that is taking too long, optimize it before turning it back on..
     ##############################
     #verbose = getlines("lsusb -s %s:%s -v"%(bus,dev))
     #for l in verbose:
     #    if l.find("bInterfaceClass")>=0:
     #        col = l.split()
     #        ifclass = " ".join(col[2:])
     ##############################

     pos = "%s:%s" % (bus,dev)
     out = "[%s] "%rgbstr(1,1,0,pos)

     assert (pos in devices)

     device = devices[pos]
     if device._model!="":
       ifclass = device._model

     if ifclass!="":
        if "hub" in device._drivers:
            out += "%s  " % rgbstr(.5,.8,.1,ifclass)
        elif "usbhid" in device._drivers:
            out += "%s  " % rgbstr(.5,.5,1,ifclass)
        elif "uvcvideo" in device._drivers or "ov534" in device._drivers:
            out += "%s  " % rgbstr(1,.3,.9,ifclass)
        elif "snd-usb-audio" in device._drivers:
            out += "%s  " % rgbstr(0.8,.2,1,ifclass)
        elif "usb-storage" in device._drivers:
            out += "%s  " % rgbstr(.7,1,1,ifclass)
        elif device._ttyname!="":
            out += "%s  " % rgbstr(1,.5,0,ifclass)
        else:
            out += "%s  " % rgbstr(.7,.8,.7,ifclass)

     out += "spd<%s> "%device.speed.decode('utf-8')

     if len(device._drivers)>0:
      out += "drv<"
      index = 0
      count = len(device._drivers)
      for drv in device._drivers:
        out += "%s" % rgbstr(1,1,1,drv)
        if count>1 and index<(count-1):
          out += " "
        index += 1

      out += "> "

     out += "product<%s " % rgbstr(.9,.6,.7,device.vendorShort())
     out += "%s> " % rgbstr(.7,.7,.8,device.modelShort())

     if device._ttyname!="":
        out += "tty<%s> " % rgbstr(1,.7,.3,device._ttyname)
     if device._serial!="":
        out += "sn<%s> " % rgbstr(1,1,.5,device._serial)

     byloc["%s:%s"%(bus,dev)]=out
     print(".",end="",flush=True)

print("")
sitems = sorted(byloc.keys(),reverse=False)
prv_bus = ""
for item in sitems:
    bus = item[0:3]
    if bus!=prv_bus:
       prv_bus = bus
       print()
    print(byloc[item])



#!/usr/bin/env python3

import argparse
from ork import macos, path

parser = argparse.ArgumentParser(description='show librarys referenced by macho binary')
parser.add_argument('--machobin', help='macho file to show libs on' )

_args = vars(parser.parse_args())

if _args["machobin"]!=None:
	machobin = _args["machobin"]
	print(macos.macho_get_id(machobin))



#!/usr/bin/env python3

from ork import path, pathtools, utils, deco
import os, argparse, sys
deco = deco.Deco()

sbase = path.zephyr_base()/"samples"

samples = {
    "helloworld": sbase/"hello_world",
    "synchro": sbase/"synchronization",
    "cppsynchro": sbase/"cpp_synchronization",
    "ethernet": sbase/"net"/"eth_native_posix",
    "getline": sbase/"subsys"/"console"/"getline",
    #"blinky": sbase/"basic"/"blink_led",
    #"rgbled": sbase/"basic"/"rgb_led",
    #"gpio": sbase/"drivers"/"gpio"
    #"servo": sbase/"basic"/"servo_motor",
    #"threads": sbase/"basic"/"threads",
}

parser = argparse.ArgumentParser(description='zephry sample builder')
parser.add_argument('--list', action="store_true" )
parser.add_argument("--build", metavar="samplename")


args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)


if args["list"]==True:
    for key in samples:
        val = samples[key]
        print("%s: %s"%(deco.key(key),deco.path((val))))
elif args["build"]!=None:
    key = args["build"]
    if key in samples:
      build_dir = path.builds()/("litexzephyrsample-%s"%key)
      src_dir = samples[key]
      pathtools.mkdir(build_dir,clean=True)
      pathtools.chdir(build_dir)
      os.system("cmake -DBOARD=litex_vexriscv %s"%str(src_dir))
      os.system("make -j %d"%utils.num_cores)
    else:
      print("unknown sample<%s>"%key)



#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import ork.target

parser = argparse.ArgumentParser(description='ork.build target information')
parser.add_argument('target', metavar='T', type=str, help='a target you want information on')

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

targetid = _args["target"]

target_arch = targetid.split('-')[0]
target_os = targetid.split('-')[1]

T = ork.target.descriptor(target_arch,target_os)

SDK = T.sdk  

from ork import dep, path
from ork.deco import Deco
deco = Deco()

target_id = "%s-%s" % (T.architecture,T.os)

def print_item(key,val):
 dstr = deco.inf(target_id)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

print_item("identifier",T.identifier)
print_item("c_compiler",SDK.c_compiler)
print_item("cxx_compiler",SDK.cxx_compiler)



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################


import os, sys, pathlib, argparse, multiprocessing, json

as_main = (__name__ == '__main__')

Path = pathlib.Path

curwd = Path(os.getcwd())

parser = argparse.ArgumentParser(description='ork.build environment creator')
parser.add_argument('--stagedir', metavar="createdir", help='create staging folder and enter session' )
parser.add_argument('--wipe', action="store_true", help='wipe old staging folder' )
parser.add_argument('--prompt', metavar="prompt", help='prompt suffix' )
parser.add_argument("--numcores", metavar="numcores", help="numcores for environment")
parser.add_argument("--quiet", action="store_true", help="no output")
parser.add_argument('--novars', action="store_true", help='do not set env vars' )
parser.add_argument('--compose',action='append',help='compose obt project into container')
parser.add_argument('--obttrace',action="store_true",help='enable OBT buildtrace logging')
parser.add_argument('--sshkey',metavar="sshkey",help='ssh key to use with OBT/GIT')
parser.add_argument('--projectdir',metavar="projectdir",help='sidechain project directory(with obt manifest)')

args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###########################################
IsQuiet = (args["quiet"]==True)
###########################################

if IsQuiet:
    os.environ["OBT_QUIET"]="1"
#else:
#    os.environ["OBT_QUIET"]=IsQuiet

def my_log(x):
    if False==IsQuiet:
       print(x)

###########################################

file_path = os.path.realpath(__file__)
my_log(file_path)
file_dir = os.path.dirname(file_path)
par2_dir = os.path.dirname(file_dir)
par3_dir = os.path.dirname(par2_dir)
par4_dir = os.path.dirname(par3_dir)
par5_dir = os.path.dirname(par4_dir)

root_dir = Path(par2_dir)
scripts_dir = root_dir/"scripts"
sys.path.append(str(scripts_dir))
project_dir = root_dir

###########################################
# are we standalone or attached to a project ?
###########################################

explicitprojectdir = None
if args["projectdir"]!=None:
  explicitprojectdir = Path(args["projectdir"])

####################
# first try implicit projectdir (curwd)
####################

try_project_manifest = curwd/"obt.project"/"obt.manifest"
if try_project_manifest.exists():
  project_dir = curwd
else:

  project_dir = root_dir/".." # default is parent of OBT 

  if (explicitprojectdir!=None) and explicitprojectdir.exists(): # explicit passed in...
    try_project_manifest = explicitprojectdir/"obt.project"/"obt.manifest"
    if try_project_manifest.exists():
      project_dir = explicitprojectdir
    else:
      assert(False) # explicit does not exist
      
###########################################

ORK_PROJECT_NAME = "obt"
if "ORK_PROJECT_NAME" in os.environ:
  ORK_PROJECT_NAME = os.environ["ORK_PROJECT_NAME"]
OBT_STAGE = curwd/".staging"
if "OBT_STAGE" in os.environ:
  OBT_STAGE = Path(os.environ["OBT_STAGE"])
if args["stagedir"]!=None:
  try_staging = Path(args["stagedir"]).resolve()
  if try_staging.exists() and args["wipe"]==False:
    print("Not going to wipe your staging folder<%s> unless you ask... use --wipe"%try_staging)
    sys.exit(0)
  if args["wipe"] and try_staging.exists():
    os.system( "rm -rf %s"%try_staging)
else:
  assert(False)

NumCores = multiprocessing.cpu_count()
if args["numcores"]!=None:
  NumCores = int(args["numcores"])
if "OBT_NUM_CORES" not in os.environ:
  os.environ["OBT_NUM_CORES"]=str(NumCores)

if args["prompt"]!=None:
  os.environ["OBT_USE_PROMPT_PREFIX"] = args["prompt"]
if try_staging!=None:
  OBT_STAGE = try_staging

GIT_SSH_COMMAND = None
if args["sshkey"]!=None:
  GIT_SSH_COMMAND="ssh -i %s" % args["sshkey"]

###########################################

import ork.deco
import ork.env
import ork.path
import ork.host
import ork.dep
import ork._globals as _glob
from ork.command import Command

deco = ork.deco.Deco()
bin_dir = root_dir/"bin"

if args["obttrace"]==True:
  _glob.enableBuildTracing()

##########################################

import ork._envutils 

envsetup = ork._envutils.EnvSetup(stagedir=OBT_STAGE,
                                  projectdir=project_dir,
                                  rootdir=root_dir,
                                  bindir=bin_dir,
                                  scriptsdir=scripts_dir,
                                  is_quiet=IsQuiet,
                                  git_ssh_command = GIT_SSH_COMMAND)

###########################################

if args["compose"] != None:
  for item in args["compose"]:
    envsetup.importProject(Path(item))


###########################################
if args["novars"]==False:
  envsetup.install() # sets OBT_STAGE env var (which prefix() uses)
###########################################
ork.path.prefix().mkdir(parents=True,exist_ok=False)
#############
# Create base scripts
#############
envsetup.lazyMakeDirs()
envsetup.genBashRc(try_staging/".bashrc")
envsetup.genLaunchScript(out_path=try_staging/".launch_env")
#############

#if not ork.host.IsAARCH64:
PYTHON = ork.dep.instance("python")
PYTHON.provide()
PYTHONDEFS = ork.dep.instance("pydefaults")
PYTHONDEFS.provide()



from ork import dep, path, command, docker, wget, pathtools
from ork.deco import Deco
import ork.module
import time, re, socket, os, sys
from pathlib import Path
deco = ork.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

class dockerinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self.type = docker.Type.SINGLE
      self._name = "amigadev"
      self.imagename = "obt-amigadev:latest"
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args):
      #git@github.com:bebbo/amiga-gcc.git
      #fetcher = dep.GithubFetcher(name="amigadev",
      #                            repospec="bebbo/amiga-gcc",
      #                            revision="b9ed9db219bc82a0ce570a51b260b7477dc5b75c",
      #                            recursive=False)
      #fetcher.fetch(self.builddir)
      UID = os.getuid()
      GID = os.getgid()
      os.chdir(str(this_dir))
      cmdlist = ["docker", "build"]
      cmdlist += ["--build-arg", "UID=%d"%UID]
      cmdlist += ["--build-arg", "GID=%d"%GID]
      cmdlist += [".", "-t", self.imagename]
      command.run(cmdlist)
      command.run(["docker","pull","sebastianbergmann/amitools"])
    ###############################################
    # launch docker container
    ###############################################
    def launch(self,launch_args,environment=None):
        CWD = os.getcwd()
        builddir = path.builds()/"amigadev-test1"
        sourcedir = this_dir/"test1"
        pathtools.mkdir(builddir,clean=True)
        cmd = [
          "docker", "run",
            "--mount","type=bind,source=%s,target=/home/amigadev/test1,readonly"%sourcedir,
            "--mount","type=bind,source=%s,target=/home/amigadev/test1-build"%builddir,
            "--mount","type=bind,source=%s,target=/home/amigadev/.cwd"%CWD,
        ]
        if environment!=None:
          for k in environment.keys():
            v = environment[k]
            cmd += ["-e","%s=%s"%(k,v)]
        if launch_args==None:
          cmd += [
            "-it",
            self.imagename,
            "/bin/bash"
          ]
        else:
          cmd += [
            "-it",
            self.imagename,
          ]
          cmd += launch_args[0].split()
        ####################
        command.run(cmd,do_log=True)
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "image_name": self.imagename,
      }
    ###############################################